/*
 * a0-slave.c : tests slave creation
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jul 10, 1997
 *
 * This program tests the creation of slave threads of ath0b.
 *
 * This program should run in only one node, without parameters.
 *
 * It starts two threads, which run concurrently with the main
 * thread. All threads add letters to a string. The string is
 * printed at the end, to figure what kind of scheduling
 * interleaving exists.
 */

#include <stdio.h>
#include <stdlib.h>
#include <a0b.h>


a0tMutex mut;
char string[1000];
int count = 0;

void addstr(char *s)
{
	int r;

	r = a0LockMutex(&mut);
	if (r)
		printf("bum\n");
	string[count++] = *s;
	a0UnlockMutex(&mut);
}


int fill_string(void *v)
{
	int i, j;

	printf("started fill_string with value %s.\n", v);

	for (i = 0; i < 200; i++) {
		for (j = 0; j < 10000; j++);
		addstr(v);
	}
}


main(int argc, char *argv[])
{

	a0tThread th1, th2;
	void *res;
	int i, a, b, c;

	a0Init(&argc, &argv);
	a0InitCommit();

	a0NewMutex(&mut);

	a0NewSlave(&th1, A0DefaultScheduling, A0DefaultPriority,
		   A0DefaultStack, fill_string, "A");
	a0NewSlave(&th2, A0DefaultScheduling, A0DefaultPriority,
		   A0DefaultStack, fill_string, "B");

	a0JoinSlave(&th1, &res);
	a0JoinSlave(&th2, &res);
	fill_string("C");

	string[count] = 0;

	for (i = a = b = c = 0; i < count; i++)
		switch (string[count]) {
		case 'A':
			a++;
			break;
		case 'B':
			b++;
			break;
		case 'C':
			c++;
			break;
		}
	printf("%s\nA=%d B=%d C=%d\n", string, a, b, c);
}
