/*
 * a0f-copy.c : copies DMA region
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * August 9, 1997
 *
 * This program tests the copy of DMA regions.
 *
 * This program must run in three nodes. No argument is expected.
 *
 * The node 0 is master ode and nodes 1 and 2 are slave nodes. All slave
 * nodes create a DMA region and send its ID to the master. The master
 * does a copy of one region to another and sends an acknowledge to the
 * slaves. The slaves echo the value of their DMA regions and all
 * terminate.
 */

#include <a0format.h>
#include <stdio.h>


void master_task(a0tPort port)
{
	a0tDMARegion g1, g2;
	a0tRequest req;
	int r;

	printf("receive DMA regions\n");
	a0FReceive(&port, 1, 0, &req, A0FDMARegion, &g1, 1);
	a0FReceive(&port, 2, 0, &req, A0FDMARegion, &g2, 1);
	printf("copy\n");
	a0FCopy(&g1, &req, &g2);
	printf("end\n");
	a0FSend(&port, 1, 0, &req, A0FInt, &r, 1);
	a0FSend(&port, 2, 0, &req, A0FInt, &r, 1);
}


void slave_task(a0tPort port)
{
	int table[10];
	int r;
	a0tDMARegion g;
	a0tRequest req;
	char s[80];

	for (r = 0; r < 10; r++)
		table[r] = a0SelfNode + 10;

	a0NewDMARegion(&g, A0FInt, &table[0], 10);
	printf("send DMA region\n");
	a0FSend(&port, 0, 0, &req, A0FDMARegion, &g, 1);
	printf("wait end\n");
	for (r = 0; r < 10000; r++)
		a0YieldThread();
	a0FReceive(&port, 0, 0, &req, A0FInt, &r, 1);

	for (r = 0; r < 10; r++)
		sprintf(s + 3 * r, "%2d ", table[r]);
	printf("%s\n", s);
}


int main(int argc, char *argv[])
{

	a0tPort port;

	a0Init(&argc, &argv);
	a0UseFormats();
	a0NewPort(&port, 1, 0);
	a0InitCommit();

	if (a0NodeCount != 3) {
		if (!a0SelfNode)
			printf("must be 3 nodes\n");
	} else {
		if (a0SelfNode)
			slave_task(port);
		else
			master_task(port);
	}
	a0Terminate();
	return 0;
}
