/*
 * a0f-fun.c : test function format
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jun 20, 1997
 *
 * This program tests the function formats.
 *
 * This program can run in any number of nodes. No arguments are
 * expected.
 *
 * It builds a new function format in all nodes, to send a structured
 * list. Every element of the list contains 100 integers and 17 floats.
 * The node 0 generates a list and sends it to all other nodes. They
 * receive it and calculate a total value and send back to the node 0.
 * Node 0 calculates itself the total value to compare with the values
 * received from all other nodes.
 */

#include <a0format.h>
#include <stdio.h>


a0tFormat ListElementFormat;

typedef struct _list_t {
	int data[100];
	float other[17];
	float total;
	struct _list_t *next;
} list_t;


a0tError PackList(a0tBuffer * bufferp,
		  a0tFormat * formatp, void *address, int count)
{
	list_t *elemp;
	int size = 0;
	a0tError ret;

	elemp = (list_t *) address;
	while (elemp) {
		size++;
		elemp = elemp->next;
	}

	/* printf("pk size %d\n", size); */

	ret = a0FPack(bufferp, A0FInt, &size, 1);
	if (ret)
		return ret;

	elemp = (list_t *) address;
	while (elemp) {
		ret = a0FPack(bufferp, &ListElementFormat, (void *) elemp, 1);
		if (ret)
			return ret;
		elemp = elemp->next;
	}

	/* printf("pk end (%d)\n", bufferp->size); */

	return A0ErrOk;
}


a0tError UnpackList(a0tBuffer * bufferp,
		    a0tFormat * formatp, void *address, int count)
{
	list_t *elemp, **lastp;
	int size;
	a0tError ret = A0ErrOk;

	ret = a0FUnpack(bufferp, A0FInt, &size, 1);
	if (ret)
		return ret;

	lastp = (list_t **) address;

	/* printf("unpk size %d\n", size); */

	while (size--) {
		elemp = (list_t *) malloc(sizeof(list_t));
		if (!elemp) {
			ret = A0ErrNoMem;
			break;
		}
		*lastp = elemp;
		lastp = &(elemp->next);
		ret = a0FUnpack(bufferp, &ListElementFormat, (void *) elemp, 1);
		if (ret)
			break;
	}

	*lastp = 0;
	return ret;
}


#define LIST_SIZE 4

void generate_sample(list_t * sample_list)
{
	int i, j;

	srand(time(0) * a0SelfNode);

	/* generate a random list */
	for (i = 0; i < LIST_SIZE; i++) {
		if (i == (LIST_SIZE - 1))
			sample_list[i].next = 0;
		else
			sample_list[i].next = sample_list + i + 1;

		sample_list[i].total = 0;

		for (j = 0; j < 100; j++)
			sample_list[i].total += (float) (sample_list[i].data[j] = rand() & 0xff);

		for (j = 0; j < 17; j++)
			sample_list[i].total += (sample_list[i].other[j] = 1 / (float) (rand() & 0xff));
	}
}


void calculate_total(list_t * plist, float *result)
{
	int j;
	float part;

	*result = 0;
	while (plist) {
		part = 0;
		for (j = 0; j < 100; j++)
			part += (float) plist->data[j];
		for (j = 0; j < 17; j++)
			part += plist->other[j];
		*result += part;
		plist->total = part;
		plist = plist->next;
	}
}


void print_list(list_t * plist)
{
	int i = 0;
	char str[1024];

	while (plist) {
		sprintf(str, "list %d: [%d,%d,%d,...] [%f,%f,%f,...] %f",
			i++,
			plist->data[0], plist->data[1], plist->data[2],
			plist->other[0], plist->other[1], plist->other[2],
			plist->total);
		puts(str);
		plist = plist->next;
	}
}


int main(int argc, char *argv[])
{
	a0tPort myport;
	a0tFormat ListFormat;
	/* a0tBuffer buffer; */

	a0Init(&argc, &argv);
	a0UseFormats();
	a0NewPort(&myport, 1, 0);
	{
		/* allocate just to descrite the format */
		list_t Element;
		a0tMapField field[2];

		a0SetMapField(field, 0, &Element, A0FInt, Element.data, 100);
		a0SetMapField(field, 1, &Element, A0FFloat, Element.other, 17);
		a0NewMapFormat(&ListElementFormat, sizeof(list_t), 2, field);
		/* free Element & field */
	}

	a0NewFunctionFormat(&ListFormat, PackList, UnpackList, NULL, NULL);
	a0InitCommit();

	/* a0NewBuffer(&buffer, A0SendBufferType, 8192); */

	/* init a different random seed each node */
	srand(a0SelfNode * time(NULL));

	if (!a0SelfNode) {
		list_t sample_list[LIST_SIZE];
		a0tRequest req1, req2;
		int i;
		float total;

		generate_sample(sample_list);
		for (i = 1; i < a0NodeCount; i++)
			a0FSend(&myport, i, 0, &req1,
				&ListFormat, sample_list, 0);
		/* PackList(&buffer, 0, &sample_list, 0); */
		/* for (i=1; i<a0NodeCount; i++) */
		/* 	a0SendBuffer(&myport, i, 0, &req1, &buffer); */
		/* printf("sent list:\n"); */
		/* print_list(sample_list); */
		for (i = 1; i < a0NodeCount; i++) {
			a0FReceive(&myport, i, 0, &req2, A0FFloat, &total, 1);
			printf("from %d got %f\n", i, total);
		}
		calculate_total(sample_list, &total);
		printf("total correct = %f\n", total);
	} else {
		a0tRequest req;
		list_t *received_list;
		float total;
		a0FReceive(&myport, 0, 0, &req, &ListFormat, &received_list, 0);

		/* a0ReceiveBuffer(&myport, 0, 0, &req, &buffer); */
		/* UnpackList(&buffer, 0, &received_list, 0); */
		/* printf("received list:\n"); */
		/* print_list(received_list); */
		calculate_total(received_list, &total);
		a0FSend(&myport, 0, 0, &req, A0FFloat, &total, 1);
	}

	a0Terminate();
}
