/*
 * a0f-map.c : test map format
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jun 20, 1997
 *
 * This program tests the map formats.
 *
 * This program can run in any number of nodes. No arguments are
 * expected.
 *
 * It builds a new map format in all nodes, with an integer, a
 * floating-point and some characters. It is used by all nodes to send
 * to node 0 a random integer, the node number converted to float and
 * the node hostname. Node 0 echoes all recived data to stdout.
 */

#include <a0format.h>
#include <stdio.h>


a0tPort myport;
a0tFormat myformat;

typedef struct {
	int i;
	float f;
	char tab[40];
} my_struct_t;


void define_myformat(void)
{
	a0tMapField fields[3];
	my_struct_t model;

	a0SetMapField(fields, 0, &model, A0FInt, &model.i, 1);
	a0SetMapField(fields, 1, &model, A0FFloat, &model.f, 1);
	a0SetMapField(fields, 2, &model, A0FChar, model.tab, 40);
	a0NewMapFormat(&myformat, sizeof(my_struct_t), 3, fields);
}


int main(int argc, char *argv[])
{
	int i, r;
	float f;
	a0tRequest req;
	struct my_str {
		int i;
		float f;
		char tab[40];
	} str;

	a0Init(&argc, &argv);
	a0UseFormats();
	a0NewPort(&myport, 1, 0);
	define_myformat();
	a0InitCommit();

	/* init a different random seed each node */
	srand(a0SelfNode * time(NULL));

	if (!a0SelfNode) {
		/* receive all random numbers */
		for (i = 1; i < a0NodeCount; i++) {
			a0FReceive(&myport, i, 0, &req, A0FInt, &r, 1);
			a0FReceive(&myport, i, 0, &req, A0FFloat, &f, 1);
			printf("received %d and %f from %d.\n", r, f, i);

			a0FReceive(&myport, i, 0, &req, &myformat, &str, 1);
			printf("received [%d,%f,%s]\n", str.i, str.f, str.tab);
		}
	} else {
		r = rand();
		f = a0SelfNode;
		a0FSend(&myport, 0, 0, &req, A0FInt, &r, 1);
		a0FSend(&myport, 0, 0, &req, A0FFloat, &f, 1);
		str.i = r;
		str.f = f;
		gethostname(str.tab, 39);
		str.tab[39] = 0;
		a0FSend(&myport, 0, 0, &req, &myformat, &str, 1);
	}
	a0Terminate();
}
