/*
 * Philosophers.c: This program implements the classical problem
 *                 of dining philosophers.
 *
 * by Alexandre Carissimi
 * Apache Project, IMAG, Grenoble, France
 * April, 21 1998.
 * 
 * To execute (after source on a0setup.csh flavour):
 *
 * a0run philosophers -a0n 1
 *
 * This program is based on Tanenbaum's solution given on
 * "Modern Operating System". Prentice Hall International.
 *
 */

#include	<ath0.h>
#include	<stdio.h>

#define		SEED	             234
#define		MAXRAND            10000
#define		N		       5
#define		LEFT	i==0?N-1:(i-1)%N
#define		RIGHT	         (i+1)%N
#define		THINKING	       0
#define		HUNGRY		       1
#define		EATING		       2
#define		DONE		       3

int 		state[N]; 
a0tMutex	mutex, MutexGo;
a0tSemaphore	s[N];
a0tCondition	cond;
int		Go=0, End[N];
char		status[N*2]={'H',' ','H',' ',
                             'H',' ','H',' ',
                             'H','\0'};
                               




/***********************************************************************
  
 ***********************************************************************/
 
void	test(int i)
{

	if (state[i] == HUNGRY && state[LEFT] != EATING &&
	    state[RIGHT] != EATING) {
	    state[i] = EATING;
	    *(status+2*i) = 'E';
	    printf("%s \n", status);
	    a0VSemaphore(&s[i]);/*a0YieldThread();*/
            End[i]++;
	}
	return;
}


/***********************************************************************
  
 ***********************************************************************/
 
void	put_forks(int i)
{

	a0LockMutex(&mutex);
	state[i] = THINKING;
	*(status+2*i) = 'T';
	printf("%s\n", status);
	test(LEFT);
	test(RIGHT);
	a0UnlockMutex(&mutex);
	/*a0YieldThread();*/       /* If scheduling is FIFO without preemption */
	                       /* We allow another philosopher to run      */	
	return;
}


/***********************************************************************
  
 ***********************************************************************/
 
void	take_forks(int i)
{

	a0LockMutex(&mutex);
	state[i] = HUNGRY;
	*(status+2*i) = 'H';
	printf("%s \n", status);	
	test(i);
	a0UnlockMutex(&mutex);
	a0PSemaphore(&s[i]); 	
	return;	
}


/***********************************************************************
  
 ***********************************************************************/
 
void	think_eat(void)
{
	sleep(1);
	return;		
}


/***********************************************************************
  
 ***********************************************************************/
 
a0tError Philosophers(void *arg)
{
	int i;
	
	i= (int )arg;


       /*
        * To guarante all philosophers creation before begining 
	* the dinner. 
	*/
	
	a0LockMutex(&MutexGo);
	while(Go==0)
	     a0WaitCondition(&cond, &MutexGo);
	a0UnlockMutex(&MutexGo);
		
	while (End[i]<10) {
		think_eat();        /* Philosophe goes to think          */
		take_forks(i);      /* acquire two forks or blocks       */
		think_eat();        /* Philosophe goes to eat            */
		put_forks(i);       /* put back the forks                */	
		End[i]=End[i]+1;	
	}


	*(status+2*i)='D';
	state[i] = DONE;
	
	a0ExitThread(0);
}




/***********************************************************************
                             M A I N
 ***********************************************************************/
 

int	main(int argc, char *argv[])
{
	a0tThread	ThreadId[N];
	int		i,ret;


	a0Init(&argc, &argv);
        		
	for(i = 0; i < N; i++) 
	   if (a0NewSemaphore( &s[i], 0)) {
	      printf("# Semaphore initialization error...\n");
	      a0Abort(0);
	   }

	if (a0NewMutex(&mutex) || a0NewMutex(&MutexGo)) {
	   printf("# Mutex initialization error...\n");
	   a0Abort(0);
	}
	   

       if (a0NewCondition(&cond)) {
          printf("# Condition variable initialisation error...\n");
          a0Abort(0);
       }

        printf("#\n#Creating %d Philosophers...\n", N);

	for(i = 0; i < N; i++) {
	   if (a0NewSlave(&ThreadId[i], A0DefaultScheduling,A0DefaultPriority,
	              A0DefaultStack, Philosophers, (void *)i)) {
	      printf("# a0NewSlave creation error...\n"); 
	      a0Abort(0);
	   }
	}

        printf("#\n# The dinner will begin...\n");

	a0LockMutex(&MutexGo);
	Go=1;
	a0BroadcastCondition(&cond);
	a0UnlockMutex(&MutexGo);

	for(i = 0; i < N; i++)
	   ret=a0JoinSlave(&ThreadId[i], NULL);

        printf("\n# Diner ends... All philosophers goes to sleep...\n\n\n");	   
	a0Terminate();
}
