/*
 * queens.c: This program implements the famous N-queens problems
 *           using simple Athapascan-0 calls. It is intend to test
 *           send/receive calls
 *
 * by Alexandre Carissimi
 * Apache Project, IMAG, Grenoble, France
 * April, 9 1998.
 *
 * To execute: a0run queens -a0n <N> (where N is the number of queens + 1)
 *                                     
 *  Queens         4     5     6    7    8    9    10    11    12
 *  --------------------------------------------------------------
 *  Solutions      2    10     4   40   92   352   724  2680 14200
 */


#include	<ath0.h>
#include	<stdio.h>

#define		WORKTAG		0
#define		DIETAG		1
#define		ROOT		0


int 		NroSolutions=0;
a0tPort		Port;


int	TryPlace( int iRow, int Column, int *Board)
{
     int	i;
     
     for(i = 1; i <= Column - 1; i++)
        if ((Board[Column - i] == iRow)     ||
	    (Board[Column - i] == iRow - i) ||
	    (Board[Column - i] == iRow + i) )
	   return (FALSE);

     return (TRUE);
}
 

int	PlaceQueens (int Column, int Nqueens, int *Board)
{
     int 		iRow, i;
     a0tRequest		Request;
    
    /*
     * Try to place a queen from board[0] column (initial value). Each
     * position is tested to see if it is possible. We begun trying to
     * place in the first row, if fails we incremente row, if success
     * we try to place one queen in the next columne.
     */
     
     
     for( iRow = 1; iRow <= Nqueens; iRow++) {
        Board[Column] = iRow;       /*Try place queen in (iRow,Columne) */
	if ( TryPlace(iRow, Column, Board) == TRUE) {
	   if ( Column == Nqueens ) {
	      NroSolutions++;
	      printf("%4d: ",a0SelfNode);
	      for (i = 1; i <= Nqueens; i++) 
	           printf( "%d ", Board[i]);
	      printf("\n");
	   }
	   else 
	      PlaceQueens ( Column + 1, Nqueens, Board);
        }
	Board[Column] = 0;
     }
     
     return;
}


void	QueensSlave(int Nqueens, int *Board)
{
     a0tRequest		Request;
 
     a0Receive(&Port, ROOT, WORKTAG, &Request, a0_INT, Board, Nqueens+1); 
     PlaceQueens( Board[0], Nqueens, Board);
     a0Send(&Port, ROOT, DIETAG, &Request, a0_INT, &NroSolutions, 1);
     
     return;
}

	      

     
void	QueensRoot (int Nqueens, int *Board)
{
    int			i, NroSol;
    a0tRequest		Request;
    
    /*
     * Send to each slave a board copy placing the first queen on
     * the (ith row;first colunme). Each slave will try place a queen
     * from the seconde colunme. Board[0] gives this information.
     */
     
     for(i=1; i < a0NodeCount; i++) {
     	 Board[0] = 2;
         Board[1] = i;
	 a0Send(&Port, i, WORKTAG, &Request, a0_INT, Board, Nqueens+1);
     }
     
    /*
     * Wait for all slaves to die. Nqueens is used only to ecomize
     * a new variable. We don't need Nqueens value no more.
     */
    
     NroSol=0;
     for(i=1; i < a0NodeCount; i++) {
        a0Receive(&Port, i, DIETAG, &Request, a0_INT, &Nqueens, 1);
	NroSol += Nqueens;
     }
     
     printf("They found %d solutions\n", NroSol);
     
     return;
}


int main(int argc, char *argv[])
{   
     int	Nqueens;
     int 	*Board;

     a0Init(&argc, &argv);
     a0NewPort(&Port, 3, A0GlobalCreation);
     a0InitCommit();
    
     if ( a0NodeCount < 5) {
        printf("The smallest problem with a non-zero solution needs\n");
	printf("4 queens... so you must launch with at least 5 nodes!!\n");
	a0DisposePort(&Port);
	a0Terminate();
	exit(0);
     }
     
     Nqueens = a0NodeCount-1;
	 
     Board = (int *) calloc(Nqueens+1, sizeof(int));
     if (Board == NULL) {
        printf("Memory allocation error on Board...\n");
	a0DisposePort(&Port);
	a0Terminate();
	exit(0);
     }
     
     
     if (a0SelfNode == ROOT)
        QueensRoot(Nqueens, Board);
     else
        QueensSlave(Nqueens, Board);
	
     free(Board);
     a0DisposePort(&Port);
     a0Terminate();
}
