/*
 *
 * _a0mpi.c : MPI-Athapascan (simple) conversions
 *
 * by Ilan Ginzburg
 * Apache Project, IMAG, Grenoble, France
 * May 29, 1996
 *
 */

#define A0_SOURCE

#include <akernel.h>
#include <stdarg.h>
#include "a0mpi.h"

/* Athapascan datatypes (equivalent to MPI's) */
a0tDatatype a0_CHAR;       
a0tDatatype a0_SHORT;
a0tDatatype a0_INT; 
a0tDatatype a0_LONG;
a0tDatatype a0_LONG_LONG_INT;
a0tDatatype a0_UNSIGNED_CHAR;
a0tDatatype a0_UNSIGNED_SHORT;
a0tDatatype a0_UNSIGNED;
a0tDatatype a0_UNSIGNED_LONG;
a0tDatatype a0_FLOAT;
a0tDatatype a0_DOUBLE;
a0tDatatype a0_LONG_DOUBLE;
a0tDatatype a0_BYTE;
a0tDatatype a0_UB;
a0tDatatype a0_LB;


/* Athapascan specific datatype(s?) */
a0tDatatype a0_PORT;

typedef void *_a0tVoidP;


/*************************************************************************
        _a0InitDatatypes - initialise the datatypes
*************************************************************************/

extern a0tError _a0InitDatatypes(void) {

  a0_CHAR = MPI_CHAR;       
  a0_SHORT = MPI_SHORT;
  a0_INT = MPI_INT; 
  a0_LONG = MPI_LONG;
/* a0_LONG_LONG_INT = MPI_LONG_LONG_INT; one day we'll support it... */
  a0_UNSIGNED_CHAR = MPI_UNSIGNED_CHAR;
  a0_UNSIGNED_SHORT = MPI_UNSIGNED_SHORT;
  a0_UNSIGNED = MPI_UNSIGNED;
  a0_UNSIGNED_LONG = MPI_UNSIGNED_LONG;
  a0_FLOAT = MPI_FLOAT;
  a0_DOUBLE = MPI_DOUBLE;
  a0_LONG_DOUBLE = MPI_LONG_DOUBLE;
  a0_BYTE = MPI_BYTE;
  a0_UB = MPI_UB;
  a0_LB = MPI_LB;

#ifdef A0_TRACE
  _a0_perf_Identify_Format(a0_CHAR, "a0_CHAR");
  _a0_perf_Identify_Format(a0_SHORT, "a0_SHORT");
  _a0_perf_Identify_Format(a0_INT, "a0_INT");
  _a0_perf_Identify_Format(a0_LONG, "a0_LONG");
/*  _a0_perf_Identify_Format(a0_LONG_LONG_INT, "a0_LONG_LONG_INT");*/
  _a0_perf_Identify_Format(a0_UNSIGNED_CHAR, "a0_UNSIGNED_CHAR");
  _a0_perf_Identify_Format(a0_UNSIGNED_SHORT, "a0_UNSIGNED_SHORT");
  _a0_perf_Identify_Format(a0_UNSIGNED, "a0_UNSIGNED");
  _a0_perf_Identify_Format(a0_UNSIGNED_LONG, "a0_UNSIGNED_LONG");
  _a0_perf_Identify_Format(a0_FLOAT, "a0_FLOAT");
  _a0_perf_Identify_Format(a0_DOUBLE, "a0_DOUBLE");
  _a0_perf_Identify_Format(a0_LONG_DOUBLE, "a0_LONG_DOUBLE");
  _a0_perf_Identify_Format(a0_BYTE, "a0_BYTE");
  _a0_perf_Identify_Format(a0_UB, "a0_UB");
  _a0_perf_Identify_Format(a0_LB, "a0_LB");
  _a0_perf_Identify_Format(a0_PORT, "a0_PORT");
#endif /* A0_TRACE */

}


/*************************************************************************
        _a0NewStructDatatype - creates a new structured datatype
*************************************************************************/

extern a0tError _a0NewStructDatatype(a0tDatatype *new, void *base,
                                        int count, ... ) {

  va_list parameters;
#ifdef __GNUC__
  a0tDatatype   oldtab[count];
  int           blltab[count];
  MPI_Aint      dsptab[count];
#else
  a0tDatatype   oldtab[A0MaxDatatypeSize];
  int           blltab[A0MaxDatatypeSize];
  MPI_Aint      dsptab[A0MaxDatatypeSize];
#endif
  void *addr;
  MPI_Aint Abase;
  int n;

  /* This function takes a list of addr, count, datatype to build a
     structure. It is used internally, to build basic datatypes.
     It returns a committed datatype. Example of usage:
     struct {
       int a;
       char b[9];
     } str;
     a0tDatatype mytype;
     _a0NewStructDatatype(&mytype, &str, 2,
                                &str.a, 1, MPI_INT
                                &str.b, 9, MPI_CHAR);
  */

  akMPI_Address(base, &Abase);

  va_start(parameters, count);

  for (n=0; n<count; n++) {
    addr = va_arg(parameters, _a0tVoidP);
    akMPI_Address(addr, dsptab+n);
    dsptab[n] -= Abase;
    blltab[n] = va_arg(parameters, int);
    oldtab[n] = va_arg(parameters, a0tDatatype);
  }

  if (akMPI_Type_struct(count, blltab, dsptab, oldtab, new))
        _a0Error(A0ErrCommType, "new str datatype str");
  if (akMPI_Type_commit(new))
        _a0Error(A0ErrCommType, "new str datatype com");

  va_end(parameters);
  return A0ErrOk;
}

