/*
 * a0parser.c
 *
 * This file has routines used to parse the command line looking for its options
 * and arguments.Basically they identify options in a simple form "-name <arg>" 
 * or a list form, like, "-name <arg1> <arg2> <arg3> ...".
 *
 * These functions DOESN'T CHANGE argc and argv arguments.
 *
 * by Alexandre Carissimi
 * Apache Project, IMAG, Grenoble, France
 * April 15,1997
 *
 * Modified on:
 *	06/04/98: includes double arguments on single and list forms
 *      21/06/98: correction on a0ArgGetDoubleList (flag was misplaced)
 *      29/06/99: inclusion of float fonctions
 *
 */


#include	<string.h>
#include	<stdlib.h>


/*
 * extern int a0ArgFindName(int argc, char **argv, char *name)
 *
 * Function: Search the ocurrence of "name" in the argument list
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 * Returns:
 *	Returns the index on argv where this argument name is given.
 * otherwise returns -1
 *
 */
 
  
extern	int a0ArgFindName(int argc, char **argv, char *name)
{
   int i;
   
   for(i=0; i<argc; i++) {
      if (strcmp(argv[i], name) == 0)
         return i;
   }
   return -1;
}



/*
 * extern int a0ArgGetStr(int argc, char **argv, char *name, char **str)
 *
 * Function: Recuperates on "str" a value  which is the argument  to the
 *           option named "name".The argument is expected to be a string          
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 * Returns:
 *      str : pointer to argument vector which has the string
 *	Returns 1 if the argument exists and it s correct, returns zero if
 *      the option isn't defined; otherwise returns -1.
 *
 */


extern int a0ArgGetStr(int argc, char **argv, char *name, char **str)
{
   int index;
   
   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
       return 0;
   
   if (index+1 >= argc)
       return -1;
       
   *str = argv[index+1];
   if (*str[0]<'A' || *str[0]>'z')
       return -1;
   return index;
}


/*
 * extern int a0ArgGetInt(int argc, char **argv, char *name, int *val)
 *
 * Function: Recuperates on "val" a value  which is the argument  to the
 *           option named "name".The argument is expected to be an integer         
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 * Returns:
 *      val : the integer value 
 *	Returns 1 if the argument exists and it is correct, otherwise
 *      returns -1
 *
 */

 
extern int a0ArgGetInt(int argc, char **argv, char *name, int *val)
{
   int  i,index;
   char *argvalue;

   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
       return 0;
   
   if (index+1 >= argc) 
       return -1;
   
   argvalue = argv[index+1];
   for(i=0; i<strlen(argvalue); i++) {
      if (argvalue[i]>='0' && argvalue[i] <= '9')
          continue;
      return -1;
   }
  
   *val = atoi (argvalue);
   return index;
}


/*
 * extern int a0ArgGetIntList(int argc, char **argv, char *name, int n, int *val)
 *
 * Function: Recuperates on "val" a sequence of "integer" which are arguments
 *           to the option named "name" in the form "-name arg1 arg2 arg3 ..."
 *          The arguments are expected to be an integer.         
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 *      n   : number of arguments to be retrieved
 * Returns:
 *      val : the integer value sequence (n sized array)
 *	Returns 1 if the arguments exist and they are correct, otherwise
 *      returns -1
 *
 *
 */
 
extern int a0ArgGetIntList(int argc, char **argv, char *name, int n, int *val)
{
   int i,j,index;
   char *argvalue;
   
   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
      return 0;
   
   if (index+n >= argc)
      return -1;
      
   for(i=0; i<n; i++) {
      argvalue = argv[index+1+i];
      for(j=0; j<strlen(argvalue); j++) {
         if (argvalue[j]>='0' && argvalue[j] <= '9')
            continue;
         return -1;
      }
      val[i] = atoi(argv[index+1+i]);
   }
   return index;
}
  
  
/*
 * extern int a0ArgGetdouble(int argc, char **argv, char *name, double *val)
 *
 * Function: Recuperates on "val" a value  which is the argument  to the
 *           option named "name".The argument is expected to be a double         
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 * Returns:
 *      val : the double value 
 *	Returns 1 if the argument exists and it is correct, otherwise
 *      returns -1
 *
 */

 
extern int a0ArgGetDouble(int argc, char **argv, char *name, double *val)
{
   int  i,index,flag=0;
   char *argvalue;

   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
       return 0;
   
   if (index+1 >= argc) 
       return -1;
   
   argvalue = argv[index+1];
   if (argvalue[0]=='-' || argvalue[0]=='+')
       i=1;
   else
       i=0;

   for(; i<strlen(argvalue); i++) {
      if (argvalue[i]>='0' && argvalue[i] <= '9')
          continue;
      if (argvalue[i]=='.' && flag==0) {
	  flag=1;
	  continue;
      }	  
      return -1;
   }
  
   *val = atof (argvalue);
   return index;
}


/*
 * extern int a0ArgGetDoubleList(int argc, char **argv, char *name, int n, double *val)
 *
 * Function: Recuperates on "val" a sequence of "double" which are arguments
 *           to the option named "name" in the form "-name arg1 arg2 arg3 ..."
 *           The arguments are expected to be an doubles.         
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 *      n   : number of arguments to be retrieved
 * Returns:
 *      val : the integer value sequence (n sized array)
 *	Returns 1 if the arguments exist and they are correct, otherwise
 *      returns -1
 *
 *
 */
 
extern int a0ArgGetDoubleList(int argc, char **argv, char *name, int n, double *val)
{ 
   int i,j,index,flag=0;
   char *argvalue;
   
   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
      return 0;
   
   if (index+n >= argc)
      return -1;
      
   for(i=0; i<n; i++) {
      flag=0;
      argvalue = argv[index+1+i];
      if (argvalue[0]=='-' || argvalue[0]=='+')
         j=1;
      else
         j=0;
     	 
      for(; j<strlen(argvalue); j++) {
         if (argvalue[j]>='0' && argvalue[j] <= '9')
            continue;
	 if (argvalue[j]=='.' && flag==0) {
	    flag=1;
	    continue;
	 }
         return -1;
      }
      val[i] = atof(argv[index+1+i]);
   }

   return index;
}



/*
 * extern int a0ArgGetFloat(int argc, char **argv, char *name, float *val)
 *
 * Function: Recuperates on "val" a value  which is the argument  to the
 *           option named "name".The argument is expected to be a double         
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 * Returns:
 *      val : the float value 
 *	Returns 1 if the argument exists and it is correct, otherwise
 *      returns -1
 *
 */

 
extern int a0ArgGetFloat(int argc, char **argv, char *name, float *val)
{
   int  i,index,flag=0;
   char *argvalue;

   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
       return 0;
   
   if (index+1 >= argc) 
       return -1;
   
   argvalue = argv[index+1];
   if (argvalue[0]=='-' || argvalue[0]=='+')
       i=1;
   else
       i=0;

   for(; i<strlen(argvalue); i++) {
      if (argvalue[i]>='0' && argvalue[i] <= '9')
          continue;
      if (argvalue[i]=='.' && flag==0) {
	  flag=1;
	  continue;
      }	  
      return -1;
   }
  
   *val = atof (argvalue);
   return index;
}


/*
 * extern int a0ArgGetFloatList(int argc, char **argv, char *name, int n, float *val)
 *
 * Function: Recuperates on "val" a sequence of "float" which are arguments
 *           to the option named "name" in the form "-name arg1 arg2 arg3 ..."
 *           The arguments are expected to be an doubles.         
 *
 * Input Parameters:
 * 	argc: argument count
 * 	argv: argument vector
 * 	name: argument name to looking for
 *      n   : number of arguments to be retrieved
 * Returns:
 *      val : the integer value sequence (n sized array)
 *	Returns 1 if the arguments exist and they are correct, otherwise
 *      returns -1
 *
 *
 */
 
extern int a0ArgGetFloatList(int argc, char **argv, char *name, int n, float *val)
{ 
   int i,j,index,flag=0;
   char *argvalue;
   
   index = a0ArgFindName(argc, argv, name);
   if (index == -1)
      return 0;
   
   if (index+n >= argc)
      return -1;
      
   for(i=0; i<n; i++) {
      flag=0;
      argvalue = argv[index+1+i];
      if (argvalue[0]=='-' || argvalue[0]=='+')
         j=1;
      else
         j=0;
     	 
      for(; j<strlen(argvalue); j++) {
         if (argvalue[j]>='0' && argvalue[j] <= '9')
            continue;
	 if (argvalue[j]=='.' && flag==0) {
	    flag=1;
	    continue;
	 }
         return -1;
      }
      val[i] = atof(argv[index+1+i]);
   }

   return index;
}


