/*
 *
 * a0threadattr.c : attributes handling 
 *
 *
 * by Alexandre Carissimi
 * Apache Project, IMAG, Grenoble, France
 * Jun 19, 1998
 *
 */

#include <akernel.h>
#include <errno.h>

#define A0_SOURCE
#include "a0error.h"
#include "a0thread.h"
#include "a0threadattr.h"


/* debugging macro */
#define a0Debug_on(a) printf a
#define a0Debug(a)


#define		_POSIX_C_SOURCE=199506L

/*************************************************************************

 *************************************************************************/
 
a0tError	_a0GetSchedPolicy( const a0tThreadAttr *attr, int *policy)
{
     int err;
     
     if ( err = akpthread_attr_getschedpolicy( attr, policy) ) 
        return(err);

     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/
 
a0tError	_a0SetSchedPolicy( a0tThreadAttr *attr, int policy)
{
     int err;
     
     if (err = akpthread_attr_setschedpolicy(attr,policy) ) 
        return(err);

     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/
 
a0tError	_a0GetSchedPriority(const a0tThreadAttr *attr, int *priority)
{
     int    err;
     struct aksched_param paramp;
     
     if ( err = akpthread_attr_getschedparam(attr,&paramp))
        return(err);
	
     *priority = paramp.aksched_priority;
     
     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/
 
a0tError	_a0SetSchedPriority( a0tThreadAttr *attr, int priority)
{
     int    err;
     struct aksched_param paramp;
     
     paramp.aksched_priority = priority;
     
     if ( err = akpthread_attr_setschedparam( attr, &paramp))
        return(err);
	
     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/
a0tError	_a0GetSchedScope( const a0tThreadAttr *attr, int *scope) 
{
     int    err;
     
     if ( err = akpthread_attr_getscope( attr, scope))
        return(err);
	
     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/

a0tError	_a0SetSchedScope( a0tThreadAttr *attr, int scope)
{
     int    err;
     
     if ( err = akpthread_attr_setscope( attr, scope))
        return(err);
	
     return(A0ErrOk);
}

/*************************************************************************

 *************************************************************************/
 
a0tError	_a0GetInherit( const a0tThreadAttr *attr, int *inherit)
{
     int    err;
     
     if ( err = akpthread_attr_getinheritsched( attr, inherit))
        return(err);
	
     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/

a0tError	_a0SetInherit( a0tThreadAttr *attr, int inherit)
{
     int    err;
     
     if ( err = akpthread_attr_setinheritsched( attr, inherit))
        return(err);
	
     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/
 
a0tError _a0SetStackSize( a0tThreadAttr *attr, size_t stacksize)
{
     int    err;
     
     if ( err = akpthread_attr_setstacksize ( attr, stacksize))
        return(err);
	
     return(A0ErrOk);
}

/*************************************************************************

 *************************************************************************/
 
a0tError _a0GetStackSize(const a0tThreadAttr *attr, size_t *stacksize)
{
     int    err;
     
     if ( err = akpthread_attr_getstacksize ( attr, stacksize))
        return(err);
	
     return(A0ErrOk);
}	

/*************************************************************************

 *************************************************************************/
 
a0tError _a0SetStackAddr( a0tThreadAttr *attr, void *stackaddr)
{
     int    err;
     
     if ( err = akpthread_attr_setstackaddr ( attr, stackaddr))
        return(err);
	
     return(A0ErrOk);
}

/*************************************************************************

 *************************************************************************/
 
a0tError _a0GetStackAddr(const a0tThreadAttr *attr, void **stackaddr)
{
     int    err;
     
     if ( err = akpthread_attr_getstackaddr ( attr, stackaddr))
        return(err);
	
     return(A0ErrOk);
}
/*************************************************************************

 *************************************************************************/

a0tError	_a0GetSchedParam( a0tThread Thread, int *policy,int *priority)
{
     int    err;
     struct aksched_param paramp;
     
     if ( err = akpthread_getschedparam( Thread.pth_id, policy, &paramp))
        return(err);
	
     *priority = paramp.aksched_priority;
     
     return(A0ErrOk);
}


/*************************************************************************

 *************************************************************************/
 

a0tError	_a0SetSchedParam( a0tThread Thread, int policy, int priority)
{
     int    err;
     struct aksched_param paramp;
     
     paramp.aksched_priority = priority;
     
     if ( err = akpthread_setschedparam( Thread.pth_id, policy, &paramp))
        return(err);
	
     
     return(A0ErrOk);
}
