/*
 *
 * a0fcopy.c : a0 copy functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 16, 1996
 *
 */

#define A0_SOURCE
#include <ath0.h>

#define A0F_SOURCE
#include "a0fmem.h"
#include "a0fwrite.h"
#include "a0fcopy.h"


#define a0Debug_on(a) printf a
#define a0Debug(a)


/******************local variables & definitions*****************/

/* copy servics */
static int _a0CopyService;


/*************************************************************************
	_a0IFCopyPart - perform a partial non-blocking DMA copy
*************************************************************************/

extern a0tError _a0IFCopyPart(a0tDMARegion *region1, int start1, int size1,
			a0tRequest *reqp,
			a0tDMARegion *region2, int start2, int size2) {

  a0tError r;
  a0tBuffer *buffer;
  int tag;
  size_t pack_siz, tot_siz;

  _a0GetPackedSize(a0_DMA_REGION, &pack_siz);
  tot_siz = 2 * pack_siz;
  _a0GetPackedSize(a0_INT, &pack_siz);
  tot_siz += 6 * pack_siz;

  if (r = _a0AllocateBuffer(&buffer, A0ThreadBufferType, tot_siz))
	return r;

  _a0FindMemTag(&tag);

  if (	(r = _a0Pack(buffer, a0_DMA_REGION, region1, 1)) ||
	(r = _a0Pack(buffer, a0_INT, &start1, 1)) ||
	(r = _a0Pack(buffer, a0_INT, &size1, 1)) ||
	(r = _a0Pack(buffer, a0_DMA_REGION, region2, 1)) ||
	(r = _a0Pack(buffer, a0_INT, &start2, 1)) ||
	(r = _a0Pack(buffer, a0_INT, &size2, 1)) ||
	(r = _a0Pack(buffer, a0_INT, &a0SelfNode, 1)) ||
	(r = _a0Pack(buffer, a0_INT, &tag, 1))		) {
	_a0DeallocateBuffer(buffer);
	return r;
  }

  a0Debug(("post copy req: node %d:%x [%d,%d] to %d:%x [%d,%d]\n",
  	region1->node, region1->data.address, start1, size1,
	region2->node, region2->data.address, start2, size2));

  r = _a0IStartRemoteThreadCallback(region1->node,
				_a0CopyService,
				A0ServiceScheduling,
				A0ServicePriority,
				A0ServiceStack,
				NULL, buffer,
				(void (*)()) _a0DeallocateBuffer,
				buffer);
  if (r) return r;

  return _a0IReceive(&_a0MemReplyPort, region2->node, tag, reqp,
			a0_INT, &_a0MemDummy, 0);
}


/*************************************************************************
	_a0FCopyPart - perform a partial blocking DMA copy
*************************************************************************/

extern a0tError _a0FCopyPart(a0tDMARegion *region1, int start1, int size1,
			a0tRequest *reqp,
			a0tDMARegion *region2, int start2, int size2) {

  a0tError r;
  a0tRequest req_aux;

  if (!reqp) reqp = &req_aux;

  if (r = _a0IFCopyPart(region1, start1, size1, reqp,
			region2, start2, size2))
	return r;

  return _a0WaitRequest(reqp);
}


/*************************************************************************
	_a0CopyServiceFunction - copy DMA server
*************************************************************************/

static a0tError _a0CopyServiceFunction(a0tBuffer *input) {

  void *address;
  int start1, size1;
  a0tDMARegion region1;
  int tag;
  a0tMemRequest message;

  a0Debug(("starting copy service\n"));

  _a0FindMemTag(&tag);

  _a0Unpack(input, a0_DMA_REGION, &region1, 1);
  _a0Unpack(input, a0_INT, &start1, 1);
  _a0Unpack(input, a0_INT, &size1, 1);
  _a0Unpack(input, a0_DMA_REGION, &message.region, 1);
  _a0Unpack(input, a0_INT, &message.region_start, 1);
  _a0Unpack(input, a0_INT, &message.region_size, 1);
  _a0Unpack(input, a0_INT, &message.reply_node, 1);
  _a0Unpack(input, a0_INT, &message.reply_tag, 1);
  message.type = A0WriteRequest;
  message.writer_node = akNode;
  message.writer_tag = tag;

  a0Debug(("got copy req r=(%d:%d) data %X format %X start %d size %d\n",
		message.reply_node, message.reply_tag, region1.data.address,
		region1.format.address, start1, size1));

  /* calculate address */
  if (_a0GetFormatOffset(region1.format.address,
			region1.data.address, start1, &address))
	_a0Error(A0ErrDaemonMsg, "copy offset");

  a0Debug(("data address=%x %.3s\n", address, address));

  if(_a0Send(&_a0MemServerPort, message.region.node, 0, NULL,
		a0_LONG_MEM_REQUEST, &message, 1))
    _a0Error(A0ErrDaemonMsg, "copy send1");

  a0Debug(("post write-cp to %d req format %x data %x start %d size %d\n",
	message.region.node, message.region.format.address,
	message.region.data.address,
	message.region_start, message.region_size));

  if (_a0FSend(&_a0MemAskPort, message.region.node, tag, NULL,
			region1.format.address,
			address, size1))
    _a0Error(A0ErrDaemonMsg, "copy send2");
}


/*************************************************************************
	_a0InitCopy - 
*************************************************************************/

extern a0tError _a0InitCopy() {

  _a0NewService(&_a0CopyService, _a0CopyServiceFunction,
		A0DefaultScheduling, A0DefaultPriority, A0DefaultStack);
}
