/*
 *
 * a0fpipe.h : a0 Pipe format functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * June 10, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0FPIPE_H
#define _A0FPIPE_H


#include <ath0.h>
#include "a0fuser.h"

#define A0PipeBufferType	(A0ReservedBufferType+2)

#define A0ErrInvPipe		(A0ErrKernel+1)

#define A0DefaultPipeSize	65536

#define A0AsyncPipe		0x000
#define A0SyncPipe		0x001
#define A0FlowPipe		0x008
#define A0OutputPipe		0x010
#define A0InputPipe		0x020

/* backward compat */
#define A0DefaultPipeProtocol	A0AsyncPipe
#define A0SendPipeProtocol	A0SendPipe
#define A0SyncPipeProtocol	A0SyncPipe
#define A0PipeInput		A0InputPipe
#define A0PipeOutput		A0OutputPipe

#define A0PipeHasData		0x040 /*buffer has data in it*/
#define A0PipeOpen		0x100
#define A0PipeError		0x200
#define A0PipeReceived		0x400


typedef struct _a0tPipe {
  a0tPort	*port;
  int	 	tag;
  int	 	node;
  a0tBuffer	buffer[1];
  size_t	initsiz;
  size_t	bufsize;
  int		flags;
  a0tRequest	request[1];
  a0tThread	daemon[1];
  a0tSemaphore	sleep[1];
  a0tBuffer	fake_buffer[1];
  a0tMutex	mutex[1];
  int		flowct;
  struct _a0tPipeQueue *queue_head;
  struct _a0tPipeQueue **queue_tail;
} a0tPipe;


a0tError _a0NewPipe(a0tPipe *newpipe,
			a0tPort *port, int node, int tag,
			size_t bufsize, int protocol, int priority);

a0tError _a0DisposePipe(a0tPipe *pipe);

a0tError _a0Flush(a0tPipe *pipe);

a0tError _a0FPut(a0tPipe *pipe, a0tRequest *request,
	a0tFormat *format, void *address, int count);

a0tError _a0FGet(a0tPipe *pipe, a0tRequest *request,
	a0tFormat *format, void *address, int count);


a0tError _a0IFPut(a0tPipe *pipe, a0tRequest *request,
			a0tFormat *format, void *address, int count);


a0tError _a0IFGet(a0tPipe *pipe, a0tRequest *request,
			a0tFormat *format, void *address, int count);


#ifdef A0_TRACE
a0tError _a0IFPutTrace(a0tPipe *pipe, a0tRequest *request,
			unsigned long *serial_nop,
			a0tFormat *format, void *address, int count);

a0tError _a0IFGetTrace(a0tPipe *pipe, a0tRequest *request,
			unsigned long *serial_nop,
			a0tFormat *format, void *address, int count);

#define _a0GetPipeThreadId(pip,idp)	(*(idp) = pip->daemon->logical_id)
#define _a0GetPipeBuffer(pip,bufp)	(*(bufp) = pip->buffer)
#endif


#ifdef A0F_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/

#endif /* A0F_SOURCE */

#endif /* _A0FPIPE_H */
