#include <a0b.h>
#include <stdio.h>

a0tPort port;

int run = 1;

#define IfError(var, txt)	\
	if (var) fprintf(stderr, "error in a0b:%d (%s)\n", var, txt)


a0tError test_th1(void *x) {
  int i;
  a0tError ret;

  while (run) {
    ret = a0Receive(&port, 0, 1, NULL, a0_INT, &i, 1);
    IfError(ret, "th1 recv");
    if (i<0) return;
    i *= 10;
    ret = a0Send(&port, 0, 4, NULL, a0_INT, &i, 1);
    IfError(ret, "th1 send");
  }
}

a0tError test_th2(void *x) {
  int i;
  a0tError ret;

  while (run) {
    ret = a0Receive(&port, 0, 2, NULL, a0_INT, &i, 1);
    IfError(ret, "th2 recv");
    if (i<0) return;
    i *= 100;
    ret = a0Send(&port, 0, 5, NULL, a0_INT, &i, 1);
    IfError(ret, "th2 send");
  }
}

a0tError test_th3(void *x) {
  int i;
  a0tError ret;

  while (run) {
    ret = a0Receive(&port, 0, 3, NULL, a0_INT, &i, 1);
    IfError(ret, "th3 recv");
    if (i<0) return;
    i *= 1000;
    ret = a0Send(&port, 0, 6, NULL, a0_INT, &i, 1);
    IfError(ret, "th3 send");
  }
}


main(int argc, char **argv) {

  a0tThread t[3];
  a0tFunction f[3] = { test_th1, test_th2, test_th3 };
  int i, tot;
  int rank, size;
  a0tError ret;
  void *res;

  /* init a different random seed each time */
  srand(time(NULL));

  ret = a0Init(&argc, &argv);
  IfError(ret, "a0Init");
  ret = a0NewPort(&port, 7, 0);
  IfError(ret, "a0NewPort");
  ret = a0InitCommit();
  IfError(ret, "a0InitCommit");

  rank = a0SelfNode;
  size = a0NodeCount;

  if (argc > 1) tot = atoi(argv[1]);
  else tot = 10;

  for (i=0; i<3; i++) {
    ret = a0NewSlave(t+i,
		A0DefaultScheduling, A0DefaultPriority, A0InheritStack,
		f[i], 0);
    IfError(ret, "new slave");
  }

  if (!rank) {
    int r, b, n, p;

    for (i=0; i<tot; i++) {
      /* chose randomly a rank, a port and a number */
      r = rand()%size;
      b = (rand()%3)+1;
      n = rand()%100;
      printf("put %d to %d:%d", n, r, b);
      /* send the number to the port of r and receives the result */
      ret = a0Send(&port, r, b, NULL, a0_INT, &n, 1);
      IfError(ret, "send main");
      ret = a0Receive(&port, r, b+3, NULL, a0_INT, &p, 1);
      IfError(ret, "send main");
      printf(" - got result #%d: %d\n", i, p);
    }
    /* terminate all threads */
    n = -1;
    for (i=0; i<a0NodeCount; i++) {
      ret = a0Send(&port, i, 1, NULL, a0_INT, &n, 1);
      IfError(ret, "send 1");
      ret = a0Send(&port, i, 2, NULL, a0_INT, &n, 1);
      IfError(ret, "send 2");
      ret = a0Send(&port, i, 3, NULL, a0_INT, &n, 1);
      IfError(ret, "send 3");
    }
  }
  else {
    /* do nothing, just wait the end */
  }

  /* join the ended threads */
  for (i=0; i<3; i++) {
    ret = a0JoinSlave(t+i, &res);
    IfError(ret, "join slave");
  }

  a0Terminate();
}
