/*
 *
 * a0fsamp9.c : Sample of a0format lock functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * July 9, 1996
 *
 */

#include <a0format.h>

typedef char string[10];

a0tPort myport;


void dump(unsigned char *p, int s) {

  char buf[1000];
  int i;

  for (i=0; i<s; i++) sprintf(buf+3*i, "%02X ", p[i]);
  printf("%s\n", buf);
}


int main(int argc, char *argv[]) {
  a0tDMARegion glo[100];
  string variab[10];
  string values[25] = { 
	"thread", "scheme", "done", "kernel", "tags", "used", "identify",
	"sending", "and/or", "receive", "impose", "that", "relation",
	"several", "messages", "same", "have", "more", "than", "pending",
	"means", "decides", "which", "satisfy", "first"};

  int i, tot, n, g1, g2, number;
  string str1, str2;
  a0tLock lock[2];

  a0Init(&argc, &argv);
  a0UseFormats();
  a0NewPort(&myport, 1, 0);
  a0InitCommit();

  /* max 10 nodes */
  if (a0SelfNode>9) {
    a0Terminate();
    exit(1);
  }

  number = a0NodeCount;
  if (number > 10) number = 10;

  /* init a different random seed each time */
  srand(a0SelfNode * time(NULL));

  /* store random values in variab */
  for (i=0; i<10; i++) {
    strcpy(variab[i], values[rand()%25]);
    printf("%s ", variab[i]);
  }
  printf("\n");

  /* create 10 local DMA regions */
  for (i=0; i<10; i++)
    a0NewDMARegion(glo+i+10*a0SelfNode, A0FChar, variab[i], 10);

  for (i=0; i<number; i++) {
    if (i==a0SelfNode) continue;
    /* tell everybody about all 10 DMA regions */
    a0IFSend(&myport, i, 0, NULL, A0FDMARegion, glo+10*a0SelfNode, 10);
    /* get all 10 DMA regions from everybody */
    a0FReceive(&myport, i, 0, NULL, A0FDMARegion, glo+(i*10), 10);
  }

  if (argc > 1) tot = atoi(argv[1]);
  else tot = 10;

  for (i=0; i<tot; i++) {
    /* chose randomly a number and a region */
    g1 = (rand()&0x7fff)%(10*number);
ali:
    g2 = (rand()&0x7fff)%(10*number);
    if (g2==g1) goto ali;
    n = (rand()&0x7fff)%25;

    a0SetLockDMARegion( lock+0, glo+g1);
    a0SetLockRead(	lock+0, A0FChar, str1, 10);
    a0SetUnlockWrite(	lock+0, A0FChar, values+n, 10);
    a0SetLockDMARegion( lock+1, glo+g2);
    a0SetLockRead(	lock+1, A0FChar, str2, 10);
    a0SetUnlockWrite(	lock+1, A0FChar, str1, 10);
    a0Lock(lock, 2, NULL);
/*
    a0Lock(lock, 1, NULL);
*/
    printf("glo[%d] get=%s, put=%s\n", g1, str1, values+n);
/*
    a0Unlock(lock, 1, NULL);
*/
    printf("glo[%d] get=%s, put=%s\n", g2, str2, str1);
    a0Unlock(lock, 2, NULL);
  }

  a0Terminate();
}
