/*
 *
 * _a0fnew.h : _a0 map functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * May 23, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0FMAP_H
#define _A0FMAP_H

#include <ath0.h>
#include <stdarg.h>
#include "a0fuser.h"


/****************types********************/

#ifdef PUT_IT_BACK_MARCELO
typedef enum {
  /* constructed map format types */
  A0ContiguousMap = A0LastFormatType + 1,
  A0VectorMap,
  A0HVectorMap,
  A0IndexedMap,
  A0HIndexedMap,
  A0StructMap,
  A0ListMap,
  A0FieldMap,

  /* basic (MPI) datatype format type */
  A0MpiDatatype
} a0tMapType;
#endif


typedef struct {
  MPI_Aint	Adisp;
  int		blocklen;
  MPI_Datatype	datatype;
} a0tMapField;


/****************variables********************/


/****************functions********************/

extern a0tError _a0SetMapField(a0tMapField *fieldp, int index,
				void *base_addr, a0tFormat *formatp,
				void *address, int count);
extern a0tError _a0NewMapFormat(a0tFormat *newformatp,
				int size,
				int count, a0tMapField *fields);


#ifdef A0F_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/


/****************types********************/

typedef struct _a0tMap {
  MPI_Datatype datatype;	/* MPI datatype created to the map */
  int status;			/* if has datatype, size, extent */
  int size;			/* size of the datatype */
  int extent;			/* extent of the map */
#ifdef PUT_IT_BACK_MARCELO
  a0tMapType maptype;		/* constructed map type */
  int count;			/* in A0Contiguous ... A0HIndexed */
  union {
    struct _a0tMap *oldmapp;	/* in A0Contiguous ... A0HIndexed */
    struct _a0tMap **oldmapptab;/* old map tab, in A0Struct */
  } umap;
  union {
    int blocklength;		/* in A0[H]Vector */
    int *blocklengthtab;	/* in A0[H]Indexed, A0Struct */
  } ubll;
  union {
    int stride;			/* in A0Vector */
    MPI_Aint Astride;		/* in A0HVector */
    int *displacementtab;	/* in A0Indexed */
    MPI_Aint *Adisplacementtab;	/* in A0HIndexed, A0Struct */
  } udsp;
#endif
} a0tMap;

/* a0tMap status bits */
#define A0MapHasNothing		0
#define A0MapHasExtent		1
#define A0MapHasDatatype	2
#define A0MapHasCommitted	4
#define A0MapHasSize		8


/****************functions********************/

extern a0tError _a0GetMapDatatype(a0tFormat *formatp,
					MPI_Datatype *datatypep);

extern a0tError _a0GetMapSize(a0tFormat *formatp,
			void *address, int count,
			size_t *sizep);

extern a0tError _a0GetMapOffset(a0tFormat *formatp,
			void *address, int count,
			void **offsetp);

#ifdef PUT_IT_BACK_MARCELO
extern a0tError _a0NewMapVaList(a0tFormat *newformatp, 
			    a0tMapType maptype, va_list parameters);
#endif

extern a0tError _a0MapPack(a0tBuffer *buffer,
		 a0tFormat *formatp, void *address, int count);
extern a0tError _a0MapUnpack(a0tBuffer *buffer,
		 a0tFormat *formatp, void *address, int count);

#endif /* A0F_SOURCE */

#endif /* _A0FMAP_H */
