/*
 *
 *  aksemap.h 
 *
 *  Semaphores on top of pthread mutexes and condition variables.
 *
 *
 *  Ilan Ginzburg.  January 29th, 1996.
 *
 */

/* Eliminates multiple includes of this file */
#ifndef _AKSEMAP_H
#define _AKSEMAP_H

/* semaphore type */
typedef struct {
   akpthread_mutex_t mutex; /* Mutex for the semaphore */
   akpthread_cond_t cond; /* Condition variable */
   int counter; /* Value of the semaphore / link to next free */
} aktSemaphore;

/* All the functions return 0 upon successful termination, and <0
 * if an error occurs. 
 * Unless otherwise specified, after an erroneous return code, the state
 * of the program is undefined and no semaphore functions should be called.
 */


/*
 * Per-semaphore functions
 */

/* Creates a semaphore with initial value of init_value. Negative values
 * are allowed (so several V are needed before a P can pass) */
extern int akSemaphoreCreate(aktSemaphore *sem, int init_value);

/* Deletes a semaphore previously created with akSemaphoreCreate. */
extern int akSemaphoreDestroy(aktSemaphore *sem);

/* Does a P on a semaphore. Blocks if need is.  */
extern int akSemaphoreP(aktSemaphore *sem);

/* Tries to do a P on a semaphore. Does not block. Returns in variable
 * suceeded if the P has succeeded (TRUE) of not (FALSE).  */
extern int akSemaphoreTryP(aktSemaphore *sem, int *succeeded);

/* Does a V on a semaphore. (and potentially wakes-up a waiting thread on
 * that semaphore).  */
extern int akSemaphoreV(aktSemaphore *sem);


#endif /* _AKSEMAP_H */
