/*
 *
 * akthread.h: DCE threads functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jun 26, 1996
 *
 * Updates:
 *
 * 18.06.98: updating POSIX defines to have all set of
 *           attributes functions. Until now some of them
 *           were not defined. (A.S. Carissimi)
 */

/* Eliminates multiple includes of this file */
#ifndef _AKTHREAD_H
#define _AKTHREAD_H

#include <pthread.h>

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#ifndef NULL
#define NULL ((void*)0)
#endif


/* supported thread personalities:
	A0_TAIX3 - AIX 3.x DCE threads
	A0_TAIX4 - AIX 4.x POSIX threads (yes, they are different!)
	A0_THP10 - HP-UX 10.x DCE threads
	A0_TMIT - MIT POSIX threads
	A0_TOSF4 - OSF 4.x POSIX threads
	A0_TSGI6 - SGI IRIX 6.x POSIX threads
	A0_TSOL2 - Solaris 2.x POSIX threads
        A0_TMAR - Marcel threads
        A0_TCRA - Cray Unicos POSIX threads
	A0_TLIN2 - Linux 2.x POSIX threads

   supported MPI personalities:
	A0_LAM - OSC LAM/MPI
	A0_MPICH - ANL MPI-CH
	A0_MPIF - IBM MPI/F
	A0_MPIBM - IBM MPI
        A0_MPIT3E - T3E MPI (from Edinburgh's CHIMP)
*/


/* set specific flags depending on the system used */
#if defined(A0_TAIX3) || defined(A0_THP10)
/* DCE threads of AIX 3 *must* reuse threads.
   Others can also if they want so. */
#define A0_REUSETHR
#endif


/* define AK_DCE if the pesonality is DCE */
#if defined(A0_TAIX3) || defined(A0_THP10)
#define AK_DCE
#endif

/* define AK_POSIX if the pesonality is POSIX */
#if defined(A0_TSOL2) || defined(A0_TAIX4) || defined(A0_TMIT) || \
    defined(A0_TSGI6)  || defined(A0_TOSF4) || defined(A0_TCRA) || \
    defined(A0_TLIN2)
#define AK_POSIX
#endif


#ifdef AK_DCE
/**********************************************************************
	This part is included for DCE THREADS
**********************************************************************/


#ifdef A0_TAIX3
#define akSETUP_THREAD "aix3"
#ifdef A0_MPIF
#define akpthread_init_np()			pthread_init()
#else
#define akpthread_init_np()
#endif
#endif


#ifdef A0_THP10
#define akSETUP_THREAD "hpux10"
#define akpthread_init_np()
#endif

#define akpthread_attr_init(a) 			pthread_attr_create(a)
#define akpthread_attr_destroy(a)		pthread_attr_delete(a)
#define akpthread_attr_setdetachstate(a, s)	0
#define akPTHREAD_CREATE_JOINABLE		0
#define akPTHREAD_CREATE_DETACHED		0
#define akpthread_attr_setscope(a, s)		0
#define akPTHREAD_SCOPE_SYSTEM			0
#define akPTHREAD_SCOPE_PROCESS			0
#define akpthread_attr_setinheritsched(a, s)	pthread_attr_setinheritsched(a, s)
#define akPTHREAD_INHERIT_SCHED			PTHREAD_INHERIT_SCHED
#define akPTHREAD_EXPLICIT_SCHED		PTHREAD_DEFAULT_SCHED
#define akpthread_attr_setschedpolicy(a, s)	pthread_attr_setsched(a, s)
#define akSCHED_FIFO				SCHED_FIFO
#define akSCHED_RR				SCHED_RR
#define akSCHED_OTHER				SCHED_OTHER
#define akpthread_attr_setschedparam(a, p)	pthread_attr_setprio(a, (p)->aksched_priority)
#define akSCHED_PRI_FIFO_MIN_NP			PRI_FIFO_MIN
#define akSCHED_PRI_FIFO_MAX_NP			PRI_FIFO_MAX
#define akSCHED_PRI_RR_MIN_NP			PRI_RR_MIN
#define akSCHED_PRI_RR_MAX_NP			PRI_RR_MAX
#define akSCHED_PRI_OTHER_MIN_NP		PRI_OTHER_MIN
#define akSCHED_PRI_OTHER_MAX_NP		PRI_OTHER_MAX
#define akpthread_attr_setstacksize(a, k)	pthread_attr_setstacksize(a, k)
#define akPTHREAD_STACK_DEFAULT_NP		pthread_attr_getstacksize(pthread_attr_default)
#define akpthread_attr_t			pthread_attr_t
#define akPTHREAD_ATTR_DEFAULT_NP		pthread_attr_default
#define akpthread_get_delta_np(d, t)		pthread_get_expiration_np(d, t)

#define akpthread_cond_init(c, a)		pthread_cond_init(c, a)
#define akpthread_cond_destroy(c)		pthread_cond_destroy(c)
#define akpthread_cond_signal(c)		pthread_cond_signal(c)
#define akpthread_cond_timedwait(c, m, t)	pthread_cond_timedwait(c, m, t)
#define akpthread_cond_wait(c, m)		pthread_cond_wait(c, m)
#define akpthread_cond_t			pthread_cond_t
#define akpthread_condattr_t			pthread_condattr_t
#define akpthread_cond_broadcast(c)		pthread_cond_broadcast(c)
#define akPTHREAD_CONDATTR_DEFAULT_NP		pthread_condattr_default

#define akpthread_mutex_init(m, a)		pthread_mutex_init(m, a)
#define akpthread_mutex_destroy(m)		pthread_mutex_destroy(m)
#define akpthread_mutex_lock(m)			pthread_mutex_lock(m)
#define akpthread_mutex_unlock(m)		pthread_mutex_unlock(m)
#define akpthread_mutex_t			pthread_mutex_t
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		pthread_mutexattr_default

#define akpthread_create(t, a, f, p)		pthread_create(t, *(a), f, p)
#define akpthread_detach(t)			pthread_detach(t)
#define akpthread_join(t, r)			pthread_join(t, r)
#define akpthread_exit(r)			pthread_exit(r)
#define aksched_yield()				pthread_yield()
#define akpthread_self()			pthread_self()
#define akpthread_equal(a,b)			pthread_equal(a,b)
#define akpthread_t				pthread_t

#define akpthread_setschedparam(t, s, p)	pthread_setscheduler(t, s, (p)->aksched_priority)
#define akpthread_getschedparam(t, s, p)	(*(s) = pthread_getscheduler(t), ((*(s)<0) ? -1 : ((p)->aksched_priority = pthread_getprio(t), (((p)->aksched_priority<0) ? -1 : 0))))
struct aksched_param {
	int aksched_priority;
};

#define akpthread_key_create(k, d)		pthread_keycreate(k, d)
#define akpthread_getspecific_np(k, v)		pthread_getspecific(k, v)
#define akpthread_setspecific(k, v)		pthread_setspecific(k, v)
#define akpthread_key_t				pthread_key_t
#endif /* AK_DCE */



#ifdef AK_POSIX
/**********************************************************************
	This part is included for POSIX THREADS
**********************************************************************/

#ifdef A0_TAIX4
#include <sys/sched.h>
#define akSETUP_THREAD "aix4"

#define akSCHED_OTHER				SCHED_OTHER
#define akSCHED_PRI_OTHER_MIN_NP		PTHREAD_PRIO_MIN
#define akSCHED_PRI_OTHER_MAX_NP		PTHREAD_PRIO_MIN

#define akPTHREAD_CONDATTR_DEFAULT_NP		NULL
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		NULL
#define akPTHREAD_CREATE_JOINABLE		PTHREAD_CREATE_UNDETACHED

/*
#define akpthread_init_np()			((akPTHREAD_ATTR_DEFAULT_NP = pthread_attr_default), pthread_attr_getstacksize(&pthread_attr_default, &akPTHREAD_STACK_DEFAULT_NP), 0)
 */
#define akpthread_init_np()			(pthread_attr_init(&akPTHREAD_ATTR_DEFAULT_NP), (akPTHREAD_STACK_DEFAULT_NP = (1024*1024)), 0)
#define aksched_yield()				pthread_yield()

#define akpthread_get_delta_np(d, t)		pthread_get_expiration_np(d, t)
#endif /* A0_TAIX4 */


#ifdef A0_TSOL2
#include <sched.h>
#include <time.h>
#define akSETUP_THREAD "sol2"

#define akSCHED_OTHER				SCHED_OTHER
#define akSCHED_PRI_OTHER_MIN_NP		1
#define akSCHED_PRI_OTHER_MAX_NP		127

#define akPTHREAD_CONDATTR_DEFAULT_NP		NULL
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		NULL
#define akPTHREAD_CREATE_JOINABLE		PTHREAD_CREATE_JOINABLE

#define akpthread_init_np()			(pthread_attr_init(&akPTHREAD_ATTR_DEFAULT_NP), (akPTHREAD_STACK_DEFAULT_NP = (1024*1024)), 0)
#define aksched_yield()				sched_yield()
extern int akpthread_get_delta_np(struct timespec *delta, struct timespec *abstime);
#endif /* A0_TSOL2 */


#ifdef A0_TMIT
#define akSETUP_THREAD "mit"

#define akSCHED_FIFO				SCHED_FIFO
#define akSCHED_PRI_FIFO_MIN_NP			PTHREAD_MIN_PRIORITY
#define akSCHED_PRI_FIFO_MAX_NP			PTHREAD_MAX_PRIORITY

#define akSCHED_RR				SCHED_RR
#define akSCHED_PRI_RR_MIN_NP			PTHREAD_MIN_PRIORITY
#define akSCHED_PRI_RR_MAX_NP			PTHREAD_MAX_PRIORITY

#define akSCHED_OTHER				SCHED_OTHER
#define akSCHED_PRI_OTHER_MIN_NP		PTHREAD_MIN_PRIORITY
#define akSCHED_PRI_OTHER_MAX_NP		PTHREAD_MAX_PRIORITY

#define akPTHREAD_CONDATTR_DEFAULT_NP		NULL
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		NULL
#define akPTHREAD_CREATE_JOINABLE		PTHREAD_CREATE_UNDETACHED

#define akpthread_init_np()			(pthread_attr_init(&akPTHREAD_ATTR_DEFAULT_NP), pthread_attr_getstacksize(&akPTHREAD_ATTR_DEFAULT_NP, &akPTHREAD_STACK_DEFAULT_NP), 0)

#define aksched_yield()				sched_yield()
#endif /* A0_TMIT */


#ifdef A0_TSGI6
#include <sched.h>
#define akSETUP_THREAD "sgi"

#define akSCHED_FIFO				SCHED_FIFO
#define akSCHED_PRI_FIFO_MIN_NP			PX_PRIO_MIN
#define akSCHED_PRI_FIFO_MAX_NP			PX_PRIO_MAX

#define akSCHED_RR				SCHED_RR
#define akSCHED_PRI_RR_MIN_NP			PX_PRIO_MIN
#define akSCHED_PRI_RR_MAX_NP			PX_PRIO_MAX

#define akPTHREAD_CONDATTR_DEFAULT_NP		NULL
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		NULL
#define akPTHREAD_CREATE_JOINABLE		PTHREAD_CREATE_UNDETACHED

#define akpthread_init_np()			(pthread_attr_init(&akPTHREAD_ATTR_DEFAULT_NP), pthread_attr_getstacksize(&akPTHREAD_ATTR_DEFAULT_NP, &akPTHREAD_STACK_DEFAULT_NP), 0)
#define aksched_yield()				sched_yield()
#endif /* A0_TSGI6 */


#ifdef A0_TOSF4
#include <sched.h>
#define akSETUP_THREAD "osf"

#define akSCHED_FIFO				SCHED_FIFO
#define akSCHED_PRI_FIFO_MIN_NP			PRI_FIFO_MIN
#define akSCHED_PRI_FIFO_MAX_NP			PRI_FIFO_MAX

#define akSCHED_RR				SCHED_RR
#define akSCHED_PRI_RR_MIN_NP			PRI_RR_MIN
#define akSCHED_PRI_RR_MAX_NP			PRI_RR_MAX

#define akSCHED_OTHER				SCHED_OTHER
#define akSCHED_PRI_OTHER_MIN_NP		PRI_OTHER_MIN
#define akSCHED_PRI_OTHER_MAX_NP		PRI_OTHER_MAX

#define akPTHREAD_CONDATTR_DEFAULT_NP		NULL
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		NULL
#define akPTHREAD_CREATE_JOINABLE		PTHREAD_CREATE_UNDETACHED

#define akpthread_init_np()			(pthread_attr_init(&akPTHREAD_ATTR_DEFAULT_NP), pthread_attr_getstacksize(&akPTHREAD_ATTR_DEFAULT_NP, &akPTHREAD_STACK_DEFAULT_NP), 0)
#define aksched_yield()				sched_yield()
#define akpthread_get_delta_np(d, t)		pthread_get_expiration_np(d, t)
#endif /* A0_TOSF4 */


#ifdef A0_TLIN2
#include <sched.h>
#include <time.h>
#define akSETUP_THREAD "lin2"

#define akSCHED_OTHER				SCHED_OTHER
#define akSCHED_PRI_OTHER_MIN_NP		1
#define akSCHED_PRI_OTHER_MAX_NP		127

#define akPTHREAD_CONDATTR_DEFAULT_NP		NULL
#define akPTHREAD_MUTEXATTR_DEFAULT_NP		NULL
#define akPTHREAD_CREATE_JOINABLE		PTHREAD_CREATE_JOINABLE

#define akpthread_init_np()			(pthread_attr_init(&akPTHREAD_ATTR_DEFAULT_NP), 0)
#define aksched_yield()				sched_yield()
#define akpthread_get_delta_np(a,b)		0
#define akpthread_attr_setstacksize(a, k)	0
#endif /* A0_TLIN2 */


/*
 *  Definitions for attributes
 */
 
#define  akpthread_attr_init(a)			pthread_attr_init(a)
#define	 akpthread_attr_destroy(a)		pthread_attr_destroy(a)
#define	 akpthread_attr_t			pthread_attr_t

#define  akpthread_attr_setscope(a,s)		pthread_attr_setscope(a,s)
#define	 akpthread_attr_getscope(a,s)           pthread_attr_getscope(a,s)
#define  akPTHREAD_SCOPE_SYSTEM                 PTHREAD_SCOPE_SYSTEM
#define  akPTHREAD_SCOPE_PROCESS                PTHREAD_SCOPE_PROCESS

#define	 akpthread_attr_setdetachstate(a,d)     pthread_attr_setdetachstate(a,d)
#define	 akpthread_attr_getdetachstate(a,d)     pthread_attr_getdetachstate(a,d)
#define  akPTHREAD_CREATE_DETACHED              PTHREAD_CREATE_DETACHED
#ifndef  akPTHREAD_CREATE_JOINABLE
#define	 akPTHREAD_CREATE_JOINABLE              PTHREAD_CREATE_JOINABLE
#endif

#ifndef A0_TLIN2
#define  akpthread_attr_setstacksize(a,s)       pthread_attr_setstacksize(a,s)
#endif
#define  akpthread_attr_getstacksize(a,s)       pthread_attr_getstacksize(a,s)
#define  aksize_t				size_t

#define  akpthread_attr_setstackaddr(a,s)       pthread_attr_setstackaddr(a,s)
#define  akpthread_attr_getstackaddr(a,s)       pthread_attr_getstackaddr(a,s)

#define  akpthread_attr_setschedparam(a,p)      pthread_attr_setschedparam(a,p)
#define  akpthread_attr_getschedparam(a,p)      pthread_attr_getschedparam(a,p)
#define  aksched_param				sched_param
#define  aksched_priority                       sched_priority

#define  akpthread_attr_setschedpolicy(a,p)     pthread_attr_setschedpolicy(a,p)
#define  akpthread_attr_getschedpolicy(a,p)     pthread_attr_getschedpolicy(a,p)

#define  akpthread_attr_setinheritsched(a,i)    pthread_attr_setinheritsched(a,i)
#define  akpthread_attr_getinheritsched(a,i)    pthread_attr_getinheritsched(a,i)
#define  akPTHREAD_INHERIT_SCHED                PTHREAD_INHERIT_SCHED
#define  akPTHREAD_EXPLICIT_SCHED               PTHREAD_EXPLICIT_SCHED

#define  akpthread_setschedparam(a,p,s)         pthread_setschedparam(a,p,s)
#define  akpthread_getschedparam(a,p,s)         pthread_getschedparam(a,p,s)

#define akpthread_cond_init(c, a)		pthread_cond_init(c, a)
#define akpthread_cond_destroy(c)		pthread_cond_destroy(c)
#define akpthread_cond_signal(c)		pthread_cond_signal(c)
#define akpthread_cond_timedwait(c, m, t)	pthread_cond_timedwait(c, m, t)
#define akpthread_cond_wait(c, m)		pthread_cond_wait(c, m)
#define akpthread_cond_t			pthread_cond_t
#define akpthread_condattr_t			pthread_condattr_t
#define akpthread_cond_broadcast(c)		pthread_cond_broadcast(c)

#define akpthread_mutex_init(m, a)		pthread_mutex_init(m, a)
#define akpthread_mutex_destroy(m)		pthread_mutex_destroy(m)
#define akpthread_mutex_lock(m)			pthread_mutex_lock(m)
#define akpthread_mutex_unlock(m)		pthread_mutex_unlock(m)
#define akpthread_mutex_trylock(m)              pthread_mutex_trylock(m)
#define akpthread_mutex_t			pthread_mutex_t

#define akpthread_create(t, a, f, p)		pthread_create(t, a, f, p)
#define akpthread_detach(t)			pthread_detach(t)
#define akpthread_join(t, r)			pthread_join(t, r)
#define akpthread_exit(r)			pthread_exit(r)
#define akpthread_self()			pthread_self()
#define akpthread_equal(a,b)			pthread_equal(a,b)
#define akpthread_t				pthread_t

#define akpthread_key_create(k, d)		pthread_key_create(k, d)
#define akpthread_getspecific_np(k, v)		(*(v) = pthread_getspecific(k), 0)
#define akpthread_setspecific(k, v)		pthread_setspecific(k, v)
#define akpthread_key_t				pthread_key_t

extern pthread_attr_t	akPTHREAD_ATTR_DEFAULT_NP;		
extern size_t		akPTHREAD_STACK_DEFAULT_NP;
#endif /* AK_POSIX */


#ifdef A0_TMAR
/**********************************************************************
This part is included for MARCEL (too different to match POSIX or DCE)
**********************************************************************/
#define akSETUP_THREAD "mar"

/* WARNING : the only effective scheduling strategie  
       is SCHED_OTHER but is named SCHED_RR */

#define akSCHED_OTHER				SCHED_RR
#define akSCHED_PRI_OTHER_MIN_NP		MIN_PRIO
#define akSCHED_PRI_OTHER_MAX_NP		MAX_PRIO

#define akpthread_attr_init(a) 			pthread_attr_init(a)
#define akpthread_attr_destroy(a)		pthread_attr_destroy(a)
#define akpthread_attr_setdetachstate(a, s)	pthread_attr_setdetachstate(a, s)
#define akPTHREAD_CREATE_JOINABLE		0
#define akPTHREAD_CREATE_DETACHED		1
#define akpthread_attr_setscope(a, s)		0
#define akPTHREAD_SCOPE_SYSTEM			0
#define akPTHREAD_SCOPE_PROCESS			0
#define akpthread_attr_setinheritsched(a, s)	0
#define akPTHREAD_INHERIT_SCHED			0
#define akPTHREAD_EXPLICIT_SCHED		0
#define akpthread_attr_setschedpolicy(a, s)	0
#define akpthread_attr_setschedparam(a, p)	pthread_attr_setprio(a, (p)->aksched_priority)
#define akpthread_attr_setstacksize(a, k)	pthread_attr_setstacksize(a, k)
#define akpthread_attr_t			pthread_attr_t
#define akPTHREAD_ATTR_DEFAULT_NP		pthread_attr_default

#define akpthread_init_np()			(pthread_init(), pthread_attr_getstacksize(&pthread_attr_default, &akPTHREAD_STACK_DEFAULT_NP))
#define aksched_yield()				pthread_trueyield_np()
#define akpthread_get_delta_np(d, t)		pthread_get_expiration_np(d, t)

#define akPTHREAD_CONDATTR_DEFAULT_NP		0
#define akpthread_cond_init(c, a)		pthread_cond_init(c, a)
#define akpthread_cond_destroy(c)		pthread_cond_destroy(c)
#define akpthread_cond_signal(c)		pthread_cond_signal(c)
#define akpthread_cond_timedwait(c, m, t)	0
#define akpthread_cond_wait(c, m)		pthread_cond_wait(c, m)
#define akpthread_cond_t			pthread_cond_t
#define akpthread_condattr_t			pthread_condattr_t
#define akpthread_cond_broadcast(c)		pthread_cond_broadcast(c)

#define akPTHREAD_MUTEXATTR_DEFAULT_NP		0
#define akpthread_mutex_init(m, a)		pthread_mutex_init(m, a)
#define akpthread_mutex_destroy(m)		pthread_mutex_destroy(m)
#define akpthread_mutex_lock(m)			pthread_mutex_lock(m)
#define akpthread_mutex_unlock(m)		pthread_mutex_unlock(m)
#define akpthread_mutex_t			pthread_mutex_t

#define akpthread_create(t, a, f, p)		pthread_create(t, a, f, p)
#define akpthread_detach(t)			pthread_detach(t)
#define akpthread_join(t, r)			pthread_join(t, r)
#define akpthread_exit(r)			pthread_exit(r)
#define akpthread_self()			pthread_self()
#define akpthread_equal(a,b)			pthread_equal(a,b)
#define akpthread_t				pthread_t

#define akpthread_setschedparam(t, s, p)	pthread_setprio(t, (p)->aksched_priority)
#define akpthread_getschedparam(t, s, p)	(((*s)=SCHED_RR), pthread_getprio(t,&((p)->aksched_priority)))

#define akpthread_key_create(k, d)		pthread_key_create(k, d)
#define akpthread_getspecific_np(k, v)		pthread_getspecific(k, v)
#define akpthread_setspecific(k, v)		pthread_setspecific(k, v)
#define akpthread_key_t				pthread_key_t

struct aksched_param {
	unsigned aksched_priority;
};

extern pthread_attr_t	pthread_attr_default;		
extern size_t		akPTHREAD_STACK_DEFAULT_NP;
#define malloc		tmalloc
#define free		tfree
#define printf		tprintf
#define fprintf		tfprintf
#endif /* A0_TMAR */


#ifdef A0_TRACE
extern int akTraceInit(void);
extern int akTraceNewThreadId(unsigned long *idp);
extern int akTraceSetThreadId(unsigned long id);
extern int akTraceGetThreadId(unsigned long *idp);
#endif /* A0_TRACE */


#endif /* _AKTHREAD_H */
