/*****************************************************************************/
/* File : a0_perf_event_macros.h                                             */
/*									     */
/* Ph. Waille		02/07/96					     */
/*****************************************************************************/

#ifndef PERF_EVENT_MACROS_H
#define PERF_EVENT_MACROS_H

typedef unsigned long long_int;	/* Used to handle all integers and addresses */
				/* Should be the longest available integer */

#include "a0_perf_timing.h"
#include "a0_perf_trace_buffer.h"
#include "a0_perf_internal_events.h"
#include "a0thread.h"

/* Inclusion needed to define the  a0_perf_trace_buffer type in the */
/* get index and put index macros */

/* Buffer overflow is only checked at event record boudaries */
/* It is therefore important to limit the size of an event record, especially
records contining character strings such as user defined events records */


#define A0_PERF_RECORD_STRING_MAX_LENGTH	48 
				/* longer strings will be truncated */

#define A0_PERF_MAX_EVENT_RECORD_LENGTH		64

/* Integer encoding */
/* To reduce event record size and solve the big versus little endianess
problem, most integers (including addresses) are encoded in a variable
length little endian format */

/* Integers are split in 7 bits chuncks in a little endian mode */
/* 8th bit in a chunk marks the last one : 1 = the last */

#define A0_PERF_LAST_BYTE	0x80


/* Macros used in tracing ATHAPASCAN routines */
/* To be use in functions whose local variable index gives the current
writing position in the trace buffer */



#define a0_perf_put_address(address)\
	a0_perf_put_integer( (long_int)((char *)address - 0))

#define a0_perf_put_integer(integer)\
	{\
	register long_int u;\
	u = (long_int) integer;\
	*a0_perf_index = u & (A0_PERF_LAST_BYTE - 1);\
	while (u >= A0_PERF_LAST_BYTE)\
	  {\
	  a0_perf_index ++;\
	  u = u >> 7;\
	  *a0_perf_index = u & (A0_PERF_LAST_BYTE - 1); \
	  }\
	*a0_perf_index++ = u | A0_PERF_LAST_BYTE;\
	}

#define a0_perf_get_integer(integer)\
	{\
	long_int	shift=0;\
	unsigned char	current_digit;\
	integer =0;\
	while ((current_digit = *a0_perf_index++) < A0_PERF_LAST_BYTE)\
	  {\
	  integer |= \
		( (long_int) current_digit) << shift;\
	  shift += 7;\
	  }\
	integer |=\
		((current_digit & (A0_PERF_LAST_BYTE - 1)) << shift);\
	}
	
#define a0_perf_put_string(string)\
	{\
	register char *string_ptr = \
		string;\
	register char *a0_perf_last_byte = \
	     a0_perf_index +  A0_PERF_RECORD_STRING_MAX_LENGTH;\
	while ( (a0_perf_index <  a0_perf_last_byte) && \
		(*a0_perf_index++ = * string_ptr++));\
	* (a0_perf_index-1) = '\0';	/* End of string */\
	}

#define a0_perf_get_string(string)\
	{\
	char *string_pointer= string;\
	while (*string_pointer++ = *a0_perf_index++);\
	}\


#define a0_perf_put_event_id(event)\
	{\
	*a0_perf_index++ = (unsigned char) event;\
	a0_perf_put_current_time;\
	}

#define a0_perf_put_time	\
	a0_perf_set_current_time;a0_perf_put_current_time

#define a0_perf_get_event_id(index,event)\
	{\
	event = *index++;\
	}


#define a0_perf_get_index 	_a0GetSelfContext (\
				&a0_perf_Trace_buffer_context,\
					 (void **)&trace_buffer);\
				a0_perf_index = trace_buffer -> index;

#define a0_perf_put_index 	trace_buffer -> index = a0_perf_index;\
				if (a0_perf_index >= (trace_buffer ->high_water_mark))\
				  {\
				  unsigned long thread_id;\
                                  _a0GetSelfThreadId (&thread_id);\
				  a0_perf_put_full_trace_buffer (trace_buffer);\
				  a0_perf_init_user_thread_trace_buffer ();\
				  }\
				return (rcode)

#define a0_perf_begin_event		char *a0_perf_index;\
					a0tError rcode;\
					a0_perf_trace_buffer *trace_buffer;\
					a0_perf_declare_current_time;\
				        a0_perf_set_current_time

/* special release for a0_perf_init : MPI not already initialized */
#define a0_perf_begin_event_init	char *a0_perf_index;\
					a0tError rcode;\
					a0_perf_trace_buffer *trace_buffer;\
					a0_perf_declare_current_time;
#define a0_perf_end_event	

#define put_info	\
			if (*file_name)\
			   {\
			   _a0_perf_IdentifyFile(file_name);\
			   *file_name=0;\
			   }\
		        _a0_perf_IdentifyEvent(source_line,file_name)

#endif
