/***************************************************************************/
/* File a0_perf_pt2pt.c                                                     */
/* Philippe Waille 05/07/96                                                */
/***************************************************************************/

#include "akcomm.h"
#include "a0_perf_pt2pt.h"
#include "a0_perf_ports.h"
#include "a0_perf_event_macros.h"
#include "a0_perf_event_codes.h"


a0tError _a0_perf_WaitRequest(a0tRequest *request)
{
a0_perf_begin_event;
rcode = _a0WaitRequest(request);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_WAITREQUEST);
a0_perf_put_address(request);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_WaitAnyRequest(a0tRequest *reqtab[], 
				 int size, int *which)
{
a0tRequest *request = NULL;
a0_perf_begin_event;
rcode = _a0WaitAnyRequest(reqtab,size,which);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_WAITANYREQUEST);
if (*which != A0NoneCompleted) request = reqtab [*which];
a0_perf_put_address (request);
a0_perf_put_integer(*which);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_TestRequest(a0tRequest *request,
                        int *result)
{
a0_perf_begin_event;
rcode = _a0TestRequest (request,result);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_TESTREQUEST);
a0_perf_put_address(request);
a0_perf_put_integer(*result);
a0_perf_put_index;
}

a0tError _a0_perf_TestAnyRequest(a0tRequest *reqtab[], 
				 int size, int *which)
{
a0tRequest *request = NULL;
a0_perf_begin_event;
rcode = _a0TestAnyRequest (reqtab,size,which);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_TESTANYREQUEST);
if (*which != A0NoneCompleted) request = reqtab [*which];
a0_perf_put_address (request);
a0_perf_put_integer(*which);
a0_perf_put_index;
}

a0tError _a0_perf_SendBuffer(a0tPort *port,
                        int dest,
                        int tag,
                        a0tRequest *request,
                        a0tBuffer  *buffer)
{
unsigned long serial_no;
a0_perf_begin_event;
/* NULL value is allowed for the request parameter */
if (request != NULL)
  {
  rcode = _a0SendBuffer (port,dest,tag,request,buffer);
  _a0GetRequestSerialNo(request,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0SendBuffer (port,dest,tag,&req,buffer);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_SENDBUFFER);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(dest);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(buffer);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_Send(a0tPort *port,
                        int dest,
                        int tag,
                        a0tRequest *request,
                        a0tDatatype datatype,
                        void *address,
                        int count)
{
unsigned long serial_no;
a0_perf_begin_event;
/* NULL value is allowed for the request parameter */
if (request != NULL)
  {
  rcode = _a0Send (port,dest,tag,request,datatype,address,count);
  _a0GetRequestSerialNo(request,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0Send (port,dest,tag,&req,datatype,address,count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_SEND);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(dest);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}



a0tError _a0_perf_ReceiveBuffer(a0tPort *port,
                        int source,
                        int tag,
                        a0tRequest *request,
                        a0tBuffer  *buffer)
{
unsigned long serial_no;
a0tRequestData reqdata;
a0_perf_begin_event;
/* NULL value is allowed for the request parameter */
if (request != NULL)
  {
  rcode = _a0ReceiveBuffer (port,source,tag,request,buffer);
  _a0GetRequestSerialNo(request,&serial_no);
  _a0GetRequestData (request,&reqdata);
  }
else
  {
  a0tRequest req;
  rcode = _a0ReceiveBuffer (port,source,tag,&req,buffer);
  _a0GetRequestSerialNo(&req,&serial_no);
  _a0GetRequestData (&req,&reqdata);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_RECEIVEBUFFER);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(reqdata.source);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(buffer);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_Receive(a0tPort *port,
                        int source,
                        int tag,
                        a0tRequest *request,
                        a0tDatatype datatype,
                        void *address,
                        int count)
{
unsigned long serial_no;
a0tRequestData reqdata;
a0_perf_begin_event;
if (request != NULL)
  {
  rcode = _a0Receive (port,source,tag,request,datatype,address,count);
  _a0GetRequestSerialNo(request,&serial_no);
  _a0GetRequestData (request,&reqdata);
  }
else
  {
  a0tRequest req;
  rcode = _a0Receive (port,source,tag,&req,datatype,address,count);
  _a0GetRequestSerialNo(&req,&serial_no);
  _a0GetRequestData (&req,&reqdata);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_RECEIVE);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(reqdata.source);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_ISendBuffer(a0tPort *port,
                        int dest,
                        int tag,
                        a0tRequest *request,
                        a0tBuffer  *buffer)
{
unsigned long serial_no;
a0_perf_begin_event;
/* NULL value is allowed for the request parameter */
if (request != NULL)
  {
  rcode = _a0ISendBuffer (port,dest,tag,request,buffer);
  _a0GetRequestSerialNo(request,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0ISendBuffer (port,dest,tag,&req,buffer);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_ISENDBUFFER);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(dest);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(buffer);
a0_perf_put_index;
}

a0tError _a0_perf_ISend(a0tPort *port,
                        int dest,
                        int tag,
                        a0tRequest *request,
                        a0tDatatype datatype,
                        void *address,
                        int count)
{
unsigned long serial_no;
a0_perf_begin_event;
/* NULL value is allowed for the request parameter */
if (request != NULL)
  {
  rcode = _a0ISend (port,dest,tag,request,datatype,address,count);
  _a0GetRequestSerialNo(request,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0ISend (port,dest,tag,&req,datatype,address,count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_ISEND);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(dest);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}



a0tError _a0_perf_IReceiveBuffer(a0tPort *port,
                        int source,
                        int tag,
                        a0tRequest *request,
                        a0tBuffer  *buffer)
{
unsigned long serial_no;
int is_any_source;
a0_perf_begin_event;
/* NULL request also allowed ! */
if (request != NULL)
  {
  rcode = _a0IReceiveBuffer (port,source,tag,request,buffer);
  _a0GetRequestSerialNo(request,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IReceiveBuffer (port,source,tag,&req,buffer);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
/* IReceiveBuffer AnySource : source info unkwon at IReceive time */
if (is_any_source = (source == A0AnySource)) {
  a0_perf_put_event_id(A0_PERF_RECORD_ID_IRECEIVEBUFFERANY);
}
else {
  a0_perf_put_event_id(A0_PERF_RECORD_ID_IRECEIVEBUFFER);
}
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
if (!is_any_source) a0_perf_put_integer(source);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(buffer);
a0_perf_put_index;
}

a0tError _a0_perf_IReceive(a0tPort *port,
                        int source,
                        int tag,
                        a0tRequest *request,
                        a0tDatatype datatype,
                        void *address,
                        int count)
{
unsigned long serial_no;
int is_any_source;
a0_perf_begin_event;
/* NULL request also allowed ! */
if (request != NULL)
  {
  rcode = _a0IReceive (port,source,tag,request,datatype,address,count);
  _a0GetRequestSerialNo(request,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IReceive (port,source,tag,&req,datatype,address,count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);		/* Free request */
  }
a0_perf_get_index;
/* IReceive AnySource : source info unkwon at IReceive time */
if (is_any_source = (source == A0AnySource)) {
  a0_perf_put_event_id(A0_PERF_RECORD_ID_IRECEIVEANY);
} else {
  a0_perf_put_event_id(A0_PERF_RECORD_ID_IRECEIVE);
}
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
if (!is_any_source) a0_perf_put_integer(source);
a0_perf_put_integer(tag);
a0_perf_put_address(request);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}


int _a0_perf_GetRequestData( a0tRequest *request, a0tRequestData *data)
{
a0_perf_begin_event;
rcode = _a0GetRequestData (request,data);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_GETREQUESTINFO);
a0_perf_put_address(request);
a0_perf_put_integer(data->source);
a0_perf_put_integer(data->size);
a0_perf_put_index;
}


a0tError _a0_perf_Probe(
			a0tPort *port,
                        int source,
                        int tag,
                        a0tRequest *request)
{
unsigned long serial_no;
a0_perf_begin_event;
rcode = _a0Probe (port,source,tag,request);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_PROBE);
a0_perf_put_integer(source);
a0_perf_put_integer(tag);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_address(request);
_a0GetRequestSerialNo(request,&serial_no);
a0_perf_put_integer(serial_no);
a0_perf_put_index;
}


a0tError _a0_perf_IProbe(
			a0tPort *port,
                        int source,
                        int tag,
                        a0tRequest *request)
{
unsigned long serial_no;
a0_perf_begin_event;
rcode = _a0IProbe (port,source,tag,request);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IPROBE);
a0_perf_put_integer(source);
a0_perf_put_integer(tag);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_address(request);
_a0GetRequestSerialNo(request,&serial_no);
a0_perf_put_integer(serial_no);
a0_perf_put_index;
}
