/***************************************************************************/
/* File a0_perf_thread.c                                                   */
/* Philippe Waille 05/07/96                                                */
/***************************************************************************/

#include "a0_perf_thread.h"
#include "a0_perf_event_macros.h"
#include "a0_perf_event_codes.h"


a0tError _a0_perf_YieldThread ()
{
a0_perf_begin_event;
rcode = _a0YieldThread();
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_YIELDTHREAD);
a0_perf_put_index;
}

a0tError _a0_perf_JoinSlave(a0tThread *thread,void **result)
{
unsigned long thread_id;
a0_perf_begin_event;
rcode = _a0JoinSlave(thread,result);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_JOINSLAVE);
_a0GetThreadId(thread,&thread_id);
a0_perf_put_integer(thread_id);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_DetachSlave(a0tThread *thread)
{
unsigned long thread_id;
a0_perf_begin_event;
rcode = _a0DetachSlave (thread);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_DETACHSLAVE);
_a0GetThreadId(thread,&thread_id);
a0_perf_put_integer(thread_id);
a0_perf_put_index;
}

a0tError _a0_perf_ExitThread (void *result)
{
a0_perf_begin_event;
rcode = _a0ExitThread (result);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_EXITTHREAD);
a0_perf_put_index;
}

a0tError _a0_perf_NewSlave(a0tThread *thread,
                          int sched,
                          int prio,
                          long  stack,
                          a0tFunction function,
                          void *arg)
{
unsigned long thread_id;
a0_perf_begin_event;
rcode = _a0NewSlave (thread,sched,prio,stack,function,arg);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_NEWSLAVE);
_a0GetThreadId(thread,&thread_id);
a0_perf_put_integer(thread_id);
a0_perf_put_integer(sched);
a0_perf_put_integer(prio);
a0_perf_put_integer(stack);
a0_perf_put_index;
}

a0tError _a0_perf_NewSemaphore(a0tSemaphore *semaphore,int value)
{
a0_perf_begin_event;
rcode = _a0NewSemaphore(semaphore, value);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_NEWSEMAPHORE);
a0_perf_put_address(semaphore);
a0_perf_put_integer(value);
a0_perf_put_index;
}

a0tError _a0_perf_DisposeSemaphore(a0tSemaphore *semaphore)
{
a0_perf_begin_event;
_a0DisposeSemaphore (semaphore);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_DISPOSESEMAPHORE);
a0_perf_put_address(semaphore);
a0_perf_put_index;
}

a0tError _a0_perf_PSemaphore(a0tSemaphore *semaphore)
{
a0_perf_begin_event;
rcode = _a0PSemaphore(semaphore);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_PSEMAPHORE);
a0_perf_put_address(semaphore);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_TryPSemaphore(a0tSemaphore *semaphore,int *succeed)
{
a0_perf_begin_event;
rcode = _a0TryPSemaphore(semaphore,succeed);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_TRYPSEMAPHORE);
a0_perf_put_address(semaphore);
a0_perf_put_integer(*succeed);
a0_perf_put_index;
}

a0tError _a0_perf_VSemaphore(a0tSemaphore *semaphore)
{
a0_perf_begin_event;
rcode = _a0VSemaphore(semaphore);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_VSEMAPHORE);
a0_perf_put_address(semaphore);
a0_perf_put_index;
}

a0tError _a0_perf_NewMutex(a0tMutex *mutex)
{
a0_perf_begin_event;
rcode = _a0NewMutex (mutex);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_NEWMUTEX);
a0_perf_put_address(mutex);
a0_perf_put_index;
}

a0tError _a0_perf_DisposeMutex(a0tMutex *mutex)
{
a0_perf_begin_event;
rcode = _a0DisposeMutex (mutex);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_DISPOSEMUTEX);
a0_perf_put_address(mutex);
a0_perf_put_index;
}

a0tError _a0_perf_LockMutex(a0tMutex *mutex)
{
a0_perf_begin_event;
rcode = _a0LockMutex (mutex);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_LOCKMUTEX);
a0_perf_put_address(mutex);
a0_perf_put_time;
a0_perf_put_index;
}

a0tError _a0_perf_UnlockMutex(a0tMutex *mutex)
{
a0_perf_begin_event;
rcode = _a0UnlockMutex (mutex);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_UNLOCKMUTEX);
a0_perf_put_address(mutex);
a0_perf_put_index;
}

extern a0tError _a0_perf_NewContext(a0tContext *context)
{
a0_perf_begin_event;
rcode = _a0NewContext(context);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_NEWCONTEXT);
a0_perf_put_address(context);
a0_perf_put_index;
}

extern a0tError _a0_perf_GetSelfContext(a0tContext *context,void **data)
{
a0_perf_begin_event;
rcode = _a0GetSelfContext(context,data);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_GETSELFCONTEXT);
a0_perf_put_address(context);
a0_perf_put_address(*data);
a0_perf_put_index;
}

extern a0tError _a0_perf_SetSelfContext(a0tContext *context,void *data)
{
a0_perf_begin_event;
rcode = _a0SetSelfContext(context,data);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_SETSELFCONTEXT);
a0_perf_put_address(context);
a0_perf_put_address(data);
a0_perf_put_index;
}


