/*
 *
 * _a0error.h : Error handling functions
 *
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 16, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0ERROR_H
#define _A0ERROR_H


typedef int a0tError;

/* a0b error return codes */
#define A0ErrOk		0
#define A0ErrNotImplem	(-1)
#define A0ErrNoMem	(-2)
#define A0ErrFailPack	(-3)
#define A0ErrLockedMutex (-4)
#define A0ErrInvFormat	(-5)
#define A0ErrInvMutex	(-6)
#define A0ErrInvThread	(-7)
#define A0ErrInvPriority (-8)
#define A0ErrInvService (-9)
#define A0ErrInvBuffer	(-10)
#define A0ErrInvTag	(-11)
#define A0ErrInvalid	(-12)
#define A0ErrInvBuffType (-13)
#define A0ErrInvReq	(-14)
#define A0ErrInvNode	(-15)

/* akernel errors
 * The order of these errors must be the same as the order of the akernel
 * errors. Look at akerror.h and _a0error.c if you don't see what I'm talking
 * about... :-)
 */
#define A0ErrKernel	(-49)
#define A0ErrFailSend	(-50)
#define A0ErrFailRecv	(-51)
#define A0ErrFailComm	(-52)
#define A0ErrNullReq	(-53)

/* a0b internal error codes. They are signalled to the user but
  a0b keeps running. */
#define A0ErrInternal	(-1000)
#define A0ErrDaemonMsg	(-1001)
#define A0ErrMemViol	(-1002)
#define A0ErrDaemonFormat (-1003)


#ifdef A0_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/

/* a0b internal fatal error codes.
 * In case of error, Athapascan0b aborts (does not return).
 */
#define A0ErrFatal	(-2000)
#define A0ErrMutexInit	(-2001)
#define A0ErrMutexLock	(-2002)
#define A0ErrMutexUnlock (-2003)
#define A0ErrMutexDestr	(-2004)
#define A0ErrCondInit	(-2005)
#define A0ErrCondSig	(-2006)
#define A0ErrCondWait	(-2007)
#define A0ErrCondDestr	(-2008)
#define A0ErrSemapCreate (-2009)
#define A0ErrSemapP	(-2010)
#define A0ErrSemapV	(-2011)
#define A0ErrSemapDestr	(-2012)
#define A0ErrMalloc	(-2013)
#define A0ErrCommDup	(-2014)
#define A0ErrCommRank	(-2015)
#define A0ErrCommType	(-2016)
#define A0ErrDaemonThreadCreate (-2017)
#define A0ErrThreadAttr (-2018)
#define A0ErrNewFormat (-2019)
#define A0ErrDetach	(-2020)
#define A0ErrSpecific	(-2021)
#define A0ErrKernelComm	(-2022)
#define A0ErrTimedOut   A0ErrCondInit


#define _a0Error(a, b)	{return _a0ErrorPrint(__FILE__, __LINE__, a, b);}

extern a0tError _a0ErrorPrint(char *file, int line, int err, char *msg);
extern a0tError _a0KernelErrorConvert(int kerror);

#endif /* A0_SOURCE */

#endif /* _A0ERROR_H */
