/*
 *
 * _a0ports.h : ports functions and management.
 *
 * by Ilan Ginzburg
 * Apache Project, IMAG, Grenoble, France
 * May 28, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0PORTS_H
#define _A0PORTS_H

#include <akernel.h>
#include "a0error.h"


/* We need one more MPI communicators than we have nodes in the computation.
 * That way, any node allocates ports on 'his' communicator (node n allocates
 * ports on communicator n+1), and globally created ports are allocated on yet
 * another communicator (communicator 0)
 */

#ifdef A0_TRACE
#define _a0GetPortId1(p) ((p).comm)
#define _a0GetPortId2(p) ((p).basetag)
#endif


/* An Ath0 port is composed of a communicator number and a base tag.
 * (base tag + user tag) is the actual tag to use in the underlying MPI call.
 */
typedef struct _a0tPort {
  int	comm;		/* Communicator number (0..a0NodeCount+1) of the port */
  int	basetag;	/* MPI base tag for the port */
  int	maxtag;		/* maximum user tag used (minimum == 0) */
} a0tPort;


/* The port creation was confuse. This defines make it clearer.
   Local creation: each node creates its own port.
   Global creation: all nodes create together the same port. */

#define A0GlobalCreation	0
#define A0LocalCreation		1


extern MPI_Comm *_a0Communicators;

extern a0tError _a0NewPort(a0tPort *port, int tags, int type);
extern a0tError _a0DisposePort(a0tPort *port);

/***************************************************************************/

#ifdef A0_SOURCE
/* Defines only used for compiling Athapascan0 */

#endif /* A0_SOURCE */


#endif /* _A0PORTS_H */
