/*
 *
 * a0prio.h : thread handling functions
 *
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 21, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0PRIO_H
#define _A0PRIO_H


#include <akernel.h>
#include "a0error.h"
#include "a0thread.h"


/****************defines********************/

/* A0Inherit???: for Slaves and Daemons only, for inheriting the attributes
 * of the creating thread. (A0InheritScheduling is in a0tScheduling)
 */
#define A0InheritPriority	(-1)
#define A0InheritStack		(1)

/* A0Service???: for services only. When a call to _a0StartRemoteThread is done
 * with this value, it means the value for that parameter should be taken from
 * the service declaration (values passed to _a0NewRTCService when the service
 * was defined at the initialization of Athapascan).
 */
/* A0ServiceScheduling defined in a0tScheduling */
#define	A0ServicePriority	(-2)
#define A0ServiceStack		(2)

/* A0Default???: for all threads, redefine the variables. */
#define	A0DefaultScheduling	_a0DefaultScheduling
#define	A0DefaultPriority	_a0DefaultPriority
#define	A0DefaultStack		_a0DefaultStack

#define _a0SetDefaultStack(size)	(_a0DefaultStack = size)

/****************types********************/

#ifdef akSCHED_FIFO
#define A0HasFifoScheduling 1
#endif
#ifdef akSCHED_RR
#define A0HasRRScheduling 1
#endif
#ifdef akSCHED_OTHER
#define A0HasOtherScheduling 1
#endif

typedef enum {
#ifdef akSCHED_FIFO
  A0FifoScheduling = akSCHED_FIFO,
#endif
#ifdef akSCHED_RR
  A0RRScheduling = akSCHED_RR,
#endif
#ifdef akSCHED_OTHER
  A0OtherScheduling = akSCHED_OTHER,
#endif
  A0InheritScheduling = (-1),
  A0ServiceScheduling = (-2),
  A0DaemonScheduling = (-3)
} a0tScheduling;


/****************variables********************/

/* Athapascan thread default values (services and slaves/daemons) */
extern a0tScheduling	_a0DefaultScheduling; /* Scheduling (fifo, rr etc..) */
extern int		_a0DefaultPriority;
extern size_t		_a0DefaultStack; /* min stack size */

/* user priority limits:
 * Athapascan programs (users) can use priorities
 * values between 0 and a0Max???Priority.
 */
extern int A0MaxFifoPriority;       /* FIFO */
extern int A0MaxRRPriority;         /* Round Robin */
extern int A0MaxOtherPriority;      /* Other (Default) */

/* Datatype for a0tScheduling */
extern MPI_Datatype _a0SchedulingDatatype;


/****************functions********************/

extern a0tError _a0GetThreadPriority(a0tThread *thread, int *prio);
extern a0tError _a0GetSelfThreadPriority(int *prio);
extern a0tError _a0GetThreadScheduling(a0tThread *thread,
					a0tScheduling *sched);
extern a0tError _a0GetSelfThreadScheduling(a0tScheduling *sched);

extern a0tError _a0SetThreadPriority(a0tThread *thread, int prio);
extern a0tError _a0SetThreadScheduling(a0tThread *thread,
					a0tScheduling sched, int prio);
extern a0tError _a0SetSelfThreadPriority(int prio);
extern a0tError _a0SetSelfThreadScheduling(a0tScheduling sched, int prio);


#ifdef A0_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/


/* Priorities reserved for Athapascan.  */
extern int _a0PriDaemonHigh, _a0PriDaemonMed, _a0PriDaemonLow;
extern int _a0PriLowest;
extern a0tScheduling _a0DaemonScheduling;


/****************functions********************/

extern a0tError _a0User2Pthread(a0tScheduling sched, int UserPrio,
					int *Psched, int *PthreadPrio);
extern  a0tError _a0Pthread2User(int Psched, int PthreadPrio,
				a0tScheduling *sched, int *UserPrio);

extern a0tError _a0IllegalPriority(a0tScheduling sched, int prio);
extern a0tError _a0InitPrio(void);


#endif /* A0_SOURCE */

#endif /* _A0PRIO_H */
