/*
 *
 * a0services.h : user interface for defining and calling a service
 *
 * by Ilan Ginzburg, Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 29, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0SERVICES_H
#define _A0SERVICES_H

#include <akernel.h>
#include "a0error.h"
#include "a0thread.h"
#include "a0prio.h"
#include "a0buffer.h"
#include "a0pt2pt.h"


/* Type of a (user) service function.
 * (this type is NOT used by the user, but is used by the compiler to check
 * the types of the user's calls).
 * This type (a0tRemoteFunction) is a function returning a0tError and
 * having one parameter, of type (a0tBuffer *)  It's a pointer to
 * the buffer passed tho the remotely created thread.
 */

typedef a0tError (*a0tRemoteFunction)(a0tBuffer *);


/* Services registration: All calls to _a0NewService on
 * all nodes must have the same order. */

a0tError _a0NewService(int *Service, a0tRemoteFunction Code,
		        a0tScheduling Sched, int Prio, size_t StackSize);


/* BLOCKING CALLS */
/* executing a service on a remote node (in a separate thread) */
extern a0tError _a0StartRemoteThread(int node, int service,
			a0tScheduling sched, int prio, size_t stack,
			a0tRequest *requestp, a0tBuffer *buffer);

/* Do a remote request call on a remote node (no thread created there) */
extern a0tError _a0StartRemoteUrgent(int node, int service,
   			a0tRequest *requestp, a0tBuffer *buffer);

/* NON-BLOCKING CALLS */
/* executing a service on a remote node (in a separate thread) */
extern a0tError _a0IStartRemoteThread(int node, int service,
			a0tScheduling sched, int prio, size_t stack,
			a0tRequest *requestp, a0tBuffer *buffer);

/* Do a remote request call on a remote node (no thread created there) */
extern a0tError _a0IStartRemoteUrgent(int node, int service,
   			a0tRequest *requestp, a0tBuffer *buffer);


/* backward compatibility, temporary defines */
#define a0tRTCFunction 		a0tRemoteFunction
#define a0tRRFunction  		a0tRemoteFunction
#define a0NewRTCService(s,c,t,p,k) a0NewService(s,c,t,p,k)
#define a0NewRRService(s, c)	a0NewService(s,c,A0DefaultScheduling, \
					A0DefaultPriority, A0DefaultStack)


#ifdef A0_SOURCE
/***************************************************************************
 * Defines only used for compiling Athapascan0
 ***************************************************************************/

extern a0tError _a0InitServices();


/* lookup for an RTC service */
a0tError _a0LookupRTCService(int service_num,
                         a0tRTCFunction *RTCCode,
                         a0tScheduling *Sched,
                         int *Prio,
                         size_t *StackSize);

/* Lookup for an RR service */
a0tError _a0LookupRRService(int service_num,
                         a0tRRFunction *RRCode);

/* executing a service on a remote node (in a separate thread) */
extern a0tError _a0IStartRemoteThreadCallback(int node, int service,
			a0tScheduling sched, int prio, size_t stack,
			a0tRequest *requestp, a0tBuffer *buffer,
			void (*callback)(), void *callback_par);

/* Do a remote request call on a remote node (no thread created there) */
extern a0tError _a0IStartRemoteUrgentCallback(int node, int service,
   			a0tRequest *requestp, a0tBuffer *buffer,
			void (*callback)(), void *callback_par);

#define A0TerminationService (-1)

#endif /* A0_SOURCE */


#endif /* _A0SERVICES_H */
