/*
 *
 * a0thread.h : thread handling functions
 *
 *
 * by Marcelo Pasin and Ilan Ginsburg
 * Apache Project, IMAG, Grenoble, France
 * Feb 21, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0THREAD_H
#define _A0THREAD_H


#include <akernel.h>
#include "a0error.h"


/****************defines********************/

/****************types********************/

typedef a0tError	(*a0tFunction)(void *);


/* Athapascan thread descriptor. */
typedef struct _a0tThread {
  akpthread_t	pth_id; /* The 'real' pthread identifier */
  struct _a0tThreadSpecific
		*specific;
#ifdef A0_TRACE
  unsigned long	logical_id; /* Logical id for trace purposes */
#endif /* A0_TRACE */
} a0tThread;


/****************variables********************/

/* Informational variables */
/* total number of user threads in the node */
extern int a0ThreadsTotal;
/* number of threads blocked on a communication */
extern int a0ThreadsBlockedComm;
/* number of threads blocked on a semaphore, mutex or condition variable */
extern int a0ThreadsBlockedLock;


/****************functions********************/

extern a0tError _a0YieldThread( void );
extern a0tError _a0JoinThread(a0tThread *thread, void **result);
extern a0tError _a0DetachThread(a0tThread *thread);
extern a0tError _a0ExitThread(void *value);
extern void *_a0threadCleanup(void *specific_data);

extern a0tError _a0NewSlave(a0tThread *thread, int sched,
			int prio, size_t stack, a0tFunction function,
			void *value);
			

#define _a0DetachSlave	_a0DetachThread
#define _a0JoinSlave	_a0JoinThread

extern a0tError _a0GetSelfThread(a0tThread *thread);


#ifdef A0_TRACE
/* these are trace functions. Do not use. */
#define _a0GetSelfThreadId(idp)   akTraceGetThreadId(idp)
#define _a0GetThreadId(thp, idp)  (*(idp) = (thp)->logical_id)
#else
#define _a0GetSelfThreadId(idp)   (*(idp)=0)
#define _a0GetThreadId(thp, idp)  (*(idp)=0)
#endif /* A0_TRACE */


#ifdef A0_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/
typedef	akpthread_attr_t	a0tThreadAttr;

a0tError _a0InitThread();

extern a0tError _a0NewThread(a0tThread *thread, int sched,
			int prio, size_t stack,
			a0tFunction function, void *arg);

/* function to startup slaves and services */
extern a0tError _a0StartThreadPlease(a0tThread *thread,
				int sched, int prio, int stack,
				void *thread_data);


#endif /* A0_SOURCE */

#endif /* _A0THREAD_H */
