#include "a0_perf_fmem.h"
#include "../basic/a0_perf_event_codes.h"
#include "../basic/a0_perf_event_macros.h"

a0tError _a0_perf_NewDMARegion(a0tDMARegion *region,
			a0tFormat *formatp, void *pointer, int size)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
rcode = _a0NewDMARegion(region, formatp, pointer, size);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_NEWDMAREGION);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(formatp);
a0_perf_put_address(pointer);
a0_perf_put_integer(size);
a0_perf_put_index;
}


a0tError _a0_perf_IFReadPart(a0tDMARegion *region, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0IFReadPart(region, start, size, reqp, formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IFReadPart(region, start, size, &req, formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFREADPART);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(formatp);
a0_perf_put_integer(start);
a0_perf_put_integer(size);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}

a0tError _a0_perf_FReadPart(a0tDMARegion *region, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0FReadPart(region, start, size, reqp, formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0FReadPart(region, start, size, &req, formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FREADPART);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start);
a0_perf_put_integer(size);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_IFRead(a0tDMARegion *region, a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0IFRead(region, reqp, formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IFRead(region, &req, formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFREAD);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}


a0tError _a0_perf_FRead(a0tDMARegion *region, a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0FRead(region, reqp, formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0FRead(region, &req, formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FREAD);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_IFWritePart(a0tDMARegion *region, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode =_a0IFWritePart(region, start, size, reqp, 
                        formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode =_a0IFWritePart(region, start, size, &req, 
                        formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFWRITEPART);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start);
a0_perf_put_integer(size);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}


a0tError _a0_perf_FWritePart(a0tDMARegion *region, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode =_a0FWritePart(region, start, size, reqp,
                        formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode =_a0FWritePart(region, start, size, &req,
                        formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FWRITEPART);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start);
a0_perf_put_integer(size);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_IFWrite(a0tDMARegion *region, a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0IFWrite(region, reqp, formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IFWrite(region, reqp, formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFWRITE);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}


a0tError _a0_perf_FWrite(a0tDMARegion *region, a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0FWrite(region, reqp, formatp, address, count);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0FWrite(region, &req, formatp, address, count);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FWRITE);
a0_perf_put_address(formatp);
_a0GetDMARegionId1 (region,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_IFCopyPart(a0tDMARegion *region1, int start1, int size1,
			a0tRequest *reqp,
			a0tDMARegion *region2, int start2, int size2)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0IFCopyPart(region1, start1, size1, reqp, 
                       region2, start2, size2);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IFCopyPart(region1, start1, size1, &req, 
                       region2, start2, size2);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFCOPYPART);
_a0GetDMARegionId1 (region1,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region1,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start1);
a0_perf_put_integer(size1);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
_a0GetDMARegionId1 (region2,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region2,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start2);
a0_perf_put_integer(size2);
a0_perf_put_index;
}

a0tError _a0_perf_FCopyPart(a0tDMARegion *region1, int start1, int size1,
			a0tRequest *reqp,
			a0tDMARegion *region2, int start2, int size2)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0FCopyPart(region1, start1, size1, reqp,
                       region2, start2, size2);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0FCopyPart(region1, start1, size1, &req,
                       region2, start2, size2);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FCOPYPART);
_a0GetDMARegionId1 (region1,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region1,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start1);
a0_perf_put_integer(size1);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
_a0GetDMARegionId1 (region2,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region2,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_integer(start2);
a0_perf_put_integer(size2);
a0_perf_put_time;
a0_perf_put_index;
}



a0tError _a0_perf_IFCopy(a0tDMARegion *region1, 
			a0tRequest *reqp,
			a0tDMARegion *region2)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0IFCopy(region1,reqp,region2);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0IFCopy(region1,&req,region2);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFCOPY);
_a0GetDMARegionId1 (region1,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region1,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
_a0GetDMARegionId1 (region2,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region2,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_index;
}


a0tError _a0_perf_FCopy(a0tDMARegion *region1,
			a0tRequest *reqp,
			a0tDMARegion *region2)
{
unsigned long serial_no;
int region_id;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (reqp != NULL)
  {
  rcode = _a0FCopy(region1,reqp,region2);
  _a0GetRequestSerialNo(reqp,&serial_no);
  }
else
  {
  a0tRequest req;
  rcode = _a0FCopy(region1,&req,region2);
  _a0GetRequestSerialNo(&req,&serial_no);
  akNoWait (&req);
}
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FCOPY);
_a0GetDMARegionId1 (region1,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region1,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_address(reqp);
a0_perf_put_integer(serial_no);
_a0GetDMARegionId1 (region2,&region_id);
a0_perf_put_integer(region_id);
_a0GetDMARegionId2 (region2,&region_id);
a0_perf_put_integer(region_id);
a0_perf_put_time;
a0_perf_put_index;
}


