#include "a0_perf_fuser.h"
#include "../basic/a0_perf_event_codes.h"
#include "../basic/a0_perf_event_macros.h"


a0tError _a0_perf_GetFormatSize(a0tFormat *formatp, void *address, 
				int count, size_t *sizep) 
{
a0_perf_begin_event;
rcode = _a0GetFormatSize(formatp,address,count,sizep);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_GETFORMATSIZE);
a0_perf_put_address (formatp);
a0_perf_put_address (address);
a0_perf_put_integer(count);
a0_perf_put_integer(*sizep);
a0_perf_put_index;
}

a0tError _a0_perf_GetFormatOffset(a0tFormat *formatp, void *address,
                                int count, void **offsetp)
{
a0_perf_begin_event;
rcode = _a0GetFormatOffset(formatp,address,count,offsetp);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_GETFORMATOFFSET);
a0_perf_put_address (formatp);
a0_perf_put_address (address);
a0_perf_put_integer(count);
a0_perf_put_address (offsetp);
a0_perf_put_index;
}

a0tError _a0_perf_UseFormats(void) 
{
a0_perf_begin_event;
rcode = _a0UseFormats();
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_USEFORMATS);
a0_perf_put_index;
}

a0tError _a0_perf_DisposeFormat(a0tFormat *formatp)
{
a0_perf_begin_event;
rcode = _a0DisposeFormat(formatp);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_DISPOSEFORMAT);
a0_perf_put_address (formatp);
a0_perf_put_index;
}

a0tError _a0_perf_IFSend(
		a0tPort *port, int dest, int tag, a0tRequest *req,
		a0tFormat *formatp, void *address, int count) 
{
unsigned long serial_no;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (req != NULL)
  {
  rcode = _a0IFSend(port, dest, tag, req, formatp, address, count); 
  _a0GetRequestSerialNo(req,&serial_no);
  }
else
  {
  a0tRequest my_req;
  rcode = _a0IFSend(port, dest, tag, &my_req, formatp, address, count); 
  _a0GetRequestSerialNo(&my_req,&serial_no);
  akNoWait (&my_req); 
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_IFSEND);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(dest);
a0_perf_put_integer(tag);
a0_perf_put_address(req);
a0_perf_put_integer(serial_no);
a0_perf_put_address(formatp);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}


a0tError _a0_perf_FSend(
		a0tPort *port, int dest, int tag, a0tRequest *req,
		a0tFormat *formatp, void *address, int count) 
{
unsigned long serial_no;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (req != NULL)
  {
  rcode = _a0FSend(port, dest, tag, req, formatp, address, count); 
  _a0GetRequestSerialNo(req,&serial_no);
  }
else
  {
  a0tRequest my_req;
  rcode = _a0FSend(port, dest, tag, &my_req, formatp, address, count); 
  _a0GetRequestSerialNo(&my_req,&serial_no);
  akNoWait (&my_req); 
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FSEND);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(dest);
a0_perf_put_integer(tag);
a0_perf_put_address(req);
a0_perf_put_integer(serial_no);
a0_perf_put_address(formatp);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_IFReceive(
		a0tPort *port, int source, int tag, a0tRequest *req,
		a0tFormat *formatp, void *address, int count) 
{
int is_any_source;
unsigned long serial_no;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (req != NULL)
  {
  rcode = _a0IFReceive(port, source, tag, req, formatp, address, count); 
  _a0GetRequestSerialNo(req,&serial_no);
  }
else
  {
  a0tRequest my_req;
  rcode = _a0IFReceive(port, source, tag, &my_req, formatp, address, count); 
  _a0GetRequestSerialNo(&my_req,&serial_no);
  akNoWait (&my_req); 
  }
a0_perf_get_index;
/* IFReceive AnySource : source info unkwon at IFReceive time */
if (is_any_source = (source == A0AnySource))
  {
  a0_perf_put_event_id(A0_PERF_RECORD_ID_IFRECEIVEANY);
  }
else 
  {
  a0_perf_put_event_id(A0_PERF_RECORD_ID_IFRECEIVE);
  }
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
if (!is_any_source) a0_perf_put_integer(source);
a0_perf_put_integer(tag);
a0_perf_put_address(req);
a0_perf_put_integer(serial_no);
a0_perf_put_address(formatp);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}

a0tError _a0_perf_FReceive(
		a0tPort *port, int source, int tag, a0tRequest *req,
		a0tFormat *formatp, void *address, int count) 
{
int is_any_source;
unsigned long serial_no;
a0tRequestData reqdata;
a0_perf_begin_event;
/* NULL is allowed for the request parameter */
if (req != NULL)
  {
  rcode = _a0FReceive(port, source, tag, req, formatp, address, count); 
  _a0GetRequestSerialNo(req,&serial_no);
  _a0GetRequestData (req,&reqdata);
  }
else
  {
  a0tRequest my_req;
  rcode = _a0FReceive(port, source, tag, &my_req, formatp, address, count); 
  _a0GetRequestSerialNo(&my_req,&serial_no);
  _a0GetRequestData (&my_req,&reqdata);
  akNoWait (&my_req); 
  }
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FRECEIVE);
a0_perf_put_integer(_a0GetPortId1(*port));
a0_perf_put_integer(_a0GetPortId2(*port));
a0_perf_put_integer(reqdata.source);
a0_perf_put_integer(tag);
a0_perf_put_address(req);
a0_perf_put_integer(serial_no);
a0_perf_put_address(formatp);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_time;
a0_perf_put_index;
}


a0tError _a0_perf_FPack(a0tBuffer *buffer,
		 a0tFormat *formatp, void *address, int count)
{
a0_perf_begin_event;
rcode = _a0FPack(buffer, formatp, address, count);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FPACK);
a0_perf_put_address(buffer);
a0_perf_put_address(formatp);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}


a0tError _a0_perf_FUnpack(a0tBuffer *buffer,
                 a0tFormat *formatp, void *address, int count)
{
a0_perf_begin_event;
rcode = _a0FUnpack(buffer, formatp, address, count);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_FUNPACK);
a0_perf_put_address(buffer);
a0_perf_put_address(formatp);
a0_perf_put_address(address);
a0_perf_put_integer(count);
a0_perf_put_index;
}
