/*

 * a0fuser.c : _a0 Format user functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * May 23, 1996
 *
 */

#define A0_SOURCE
#include <ath0.h>
#include <stdarg.h>

#define A0F_SOURCE
#include "a0fuser.h"
#include "a0fmap.h"
#include "a0fxdr.h"


#define a0Debug(a)
#define a0Debug_on(a) printf a


/******************local variables & definitions*****************/


#define A0MaxBasicFormats 15

a0tFormat _a0BasicFormats[A0MaxBasicFormats];
static a0tMap _a0BasicMaps[A0MaxBasicFormats];
static a0tDatatype _a0BasicDatatypes[A0MaxBasicFormats] =
{
	MPI_CHAR, MPI_SHORT, MPI_INT, MPI_LONG, MPI_UNSIGNED_CHAR,
	MPI_UNSIGNED_SHORT, MPI_UNSIGNED, MPI_UNSIGNED_LONG,
	MPI_FLOAT, MPI_DOUBLE, 0, MPI_BYTE,
	0, 0, 0,
};

int _a0UseFormatsRun = 0;


#ifdef A0_TRACE
char *format_trace_names[] =
{
	"A0FChar", "A0FShort", "A0FInt", "A0FLong", "A0FUChar",
	"A0FUShort", "A0FUInt", "A0FULong",
	"A0FFloat", "A0FDouble", "A0FLDouble", "A0FByte",
	"A0FDMARegion", "A0FPort", "A0FAddress",
};
#endif				/* A0_TRACE */


/*#define DEBUG_FORMAT */
#ifdef DEBUG_FORMAT
/*************************************************************************
			print_format
*************************************************************************/

int print_format(a0tFormat * f)
{
	char *sftype[] =
	{"A0NullFormatType", "A0Map", "A0Xdr", "last",
	 "A0ContiguousMap", "A0VectorMap", "A0HVectorMap",
	 "A0IndexedMap", "A0HIndexedMap", "A0StructMap",
	 "A0ListMap", "A0MpiDatatype"};
	int i, j;
	char *sform[] =
	{"A0FChar", "A0FShort", "A0FInt", "A0FLong",
	 "A0FUChar", "A0FUShort", "A0FUInt", "A0FULong", "A0FFloat",
	 "A0FDouble", "A0FLDouble", "A0FByte", "A0FDMARegion",
	 "A0FPort", "A0FAddress", "user"};
	a0tFormat *tform[] =
	{A0FChar, A0FShort, A0FInt, A0FLong,
	 A0FUChar, A0FUShort, A0FUInt, A0FULong, A0FFloat,
	 A0FDouble, A0FLDouble, A0FByte, A0FDMARegion,
	 A0FPort, A0FAddress};

	for (j = 0;
	     (j < 14) && (tform[j]->u.mapp != f->u.mapp->umap.oldmapp);
	     j++);

	if (f->type == A0Map) {
		printf("format %s [\n  map=%X\n  count=%d\n  oldmap=%x(%s)\n"
		       "  status=%d\n  datatype=%X",
		       sftype[f->u.mapp->maptype],
		       f->u.mapp, f->u.mapp->count,
		       f->u.mapp->umap.oldmapp, sform[j],
		       f->u.mapp->status, f->u.mapp->datatype);
		switch (f->u.mapp->maptype) {
		case A0ContiguousMap:
		case A0MpiDatatype:
			break;
		case A0VectorMap:
		case A0HVectorMap:
			printf("  bll=%d\n  str=%d\n",
			       f->u.mapp->ubll.blocklength, f->u.mapp->udsp.stride);
		case A0IndexedMap:
		case A0HIndexedMap:
		case A0StructMap:
			printf("  blltab=[");
			for (i = 0; i < f->u.mapp->count; i++)
				printf("%d ", f->u.mapp->ubll.blocklengthtab[i]);
			printf("]\n  dsptab=[");
			for (i = 0; i < f->u.mapp->count; i++)
				printf("%d ", f->u.mapp->udsp.displacementtab[i]);
			if (f->u.mapp->maptype == A0StructMap) {
				printf("]\n  oldtab=[");
				for (i = 0; i < f->u.mapp->count; i++) {
					for (j = 0;
					     (j < 14) && (tform[j]->u.mapp != f->u.mapp->umap.oldmapptab[i]);
					     j++);
					printf("%X(%s) ", f->u.mapp->umap.oldmapptab[i],
					       sform[j]);
				}
			}
			printf("]\n");
		}
	}
	printf("]\n");
}
#endif


/************************************************************************
	_a0GetFormatSize - Calculates the packed size of formatted data
************************************************************************/

extern a0tError _a0GetFormatSize(a0tFormat * formatp,
				 void *address, int count,
				 size_t * sizep)
{
	/* Calculates the packed size of a given amount of data.
	   The data have the format given, start at address and have
	   count elements. */

	switch (formatp->type) {
	case A0Map:
		return _a0GetMapSize(formatp, address, count, sizep);

	case A0Xdr:
		return _a0GetXdrSize(formatp, address, count, sizep);
	}

	return A0ErrInvFormat;
}


/************************************************************************
	_a0GetFormatOffset - Calculates the offset of next element
************************************************************************/

extern a0tError _a0GetFormatOffset(a0tFormat * formatp,
				   void *address, int count,
				   void **offsetp)
{
	/* Calculates the offsetn of the n-th element of some formatted
	   data. If it's a map, the size is (count * size of one element).
	   If it's a xdr, a function must calculate it. */

	switch (formatp->type) {
	case A0Map:
		return _a0GetMapOffset(formatp, address, count, offsetp);

	case A0Xdr:
		if (formatp->u.xdrp->offsetfun)
			return formatp->u.xdrp->offsetfun(formatp,
					address, count, offsetp);
		else
			return A0ErrNotImplem;
	}

	return A0ErrInvFormat;
}


/************************************************************************
_a0GetFormatCount - Finds the count of formatted data to fit a given size
************************************************************************/

extern a0tError _a0GetFormatCount(a0tFormat * formatp,
				  void *address, int size, int *countp)
{
	/* Calculates number of packed data elements that fit in a given
	   size. The data have the format given and start at address. */

	a0tError r;
	size_t size2;
	int count, last, n;

	switch (formatp->type) {
	case A0Map:
		if (r = _a0GetMapSize(formatp, address, 1, &size2))
			return r;
		*countp = size / size2;
		return A0ErrOk;

	case A0Xdr:
		/* Xdrs can have any size, given a number of elements. Do an
		   approximation method to reach the size. */
		/* gets one's size */
		if (r = _a0GetXdrSize(formatp, address, 1, &size2))
			return r;
		count = size / size2;
		/* get size of count elements */
		if (r = _a0GetXdrSize(formatp, address, count, &size2))
			return r;

		n = last = 0;
		while ((size != size2) && ((!last) || (n < 3))) {
			if (size2 > size) {
				count = count * (size2 - size) / size2;
				if (r = _a0GetXdrSize(formatp,
						address, count, &size2))
					return r;
			}
			if (size2 < size) {
				last = count;
				count += (size - size2) / size2;
				if (r = _a0GetXdrSize(formatp,
						address, count, &size2))
					return r;
			}
			if (last)
				n++;
		}
		*countp = last;
		return A0ErrOk;
	}
}


/*************************************************************************
		_a0SetBasicFormat - set up a basic format
*************************************************************************/

extern a0tError _a0SetBasicFormat(a0tFormat * formatp,
				  a0tDatatype datatype)
{
	int n;

	/*
	 * Used to build basic formats by any module of Athapascan, as
	 * _a0DMARegion, _a0Team, etc. The datatype passed MUST be committed.
	 */

	/* find out the index of this basic format */
	n = formatp - _a0BasicFormats;
	if (n < 0 || n > A0MaxBasicFormats)
		return A0ErrInvFormat;

	formatp->type = A0Map;
	formatp->u.mapp = _a0BasicMaps + n;

#ifdef PUT_IT_BACK_MARCELO
	_a0BasicMaps[n].maptype = A0MpiDatatype;
#endif
	_a0BasicMaps[n].datatype = datatype;
	_a0BasicMaps[n].status = A0MapHasDatatype | A0MapHasCommitted;

	a0Debug(("datatype#%d: %x\n", n, datatype));

#ifdef A0_TRACE
	_a0_perf_Identify_Format(formatp,
			  format_trace_names[formatp - _a0BasicFormats]);
#endif

	return A0ErrOk;
}


/*************************************************************************
		_a0UseFormats - format module init
*************************************************************************/

extern a0tError _a0UseFormats(void)
{
	int n;

	if (_a0UseFormatsRun)
		return A0ErrOk;
	_a0UseFormatsRun = 1;

	_a0BasicDatatypes[10] = MPI_LONG_DOUBLE;

	for (n = 0; n < A0MaxBasicFormats; n++)
		_a0SetBasicFormat(_a0BasicFormats + n, _a0BasicDatatypes[n]);
	_a0SetBasicFormat(A0FPort, a0_PORT);

	_a0UseDMA();

	return A0ErrOk;
}


#ifdef PUT_IT_BACK_MARCELO
/*************************************************************************
	_a0NewFormat - creates a format
*************************************************************************/

extern a0tError _a0NewFormat(a0tFormat * newformatp,
			     int formattype,...)
{
	va_list parameters;
	int i;
	a0tError ret;

	va_start(parameters, formattype);

	a0Debug(("new format %d\n", formattype));

	/* put invalid type in the new format */
	newformatp->type = A0NullFormatType;

	switch (formattype) {
	case A0MpiDatatype:
	case A0ContiguousMap:
	case A0VectorMap:
	case A0HVectorMap:
	case A0IndexedMap:
	case A0HIndexedMap:
	case A0StructMap:
	case A0ListMap:
		ret = _a0NewMapVaList(newformatp,
			(a0tMapType) formattype, parameters);
		break;

	case A0Xdr:
		ret = _a0NewXdrVaList(newformatp, formattype, parameters);
		break;

	default:
		return A0ErrInvFormat;
	}

	va_end(parameters);
	a0Debug(("new format %d ended\n", formattype));

#ifdef DEBUG_FORMAT
	print_format(newformatp);
#endif

	return ret;
}
#endif


/*************************************************************************
	_a0DisposeFormat - disposes a format
*************************************************************************/

extern a0tError _a0DisposeFormat(a0tFormat * formatp)
{
	switch (formatp->type) {
	case A0Map:
		free(formatp->u.mapp);
		break;

	case A0Xdr:
		free(formatp->u.xdrp);
		break;

	default:
		return A0ErrInvFormat;
	}

	formatp->type = A0NullFormatType;

	return A0ErrOk;
}


/*************************************************************************
_a0IFSendCallback - Sends a formatted message (nonblocking) with callback
*************************************************************************/

extern a0tError _a0IFSendCallback(
		     a0tPort * port, int dest, int tag, a0tRequest * req,
			   a0tFormat * formatp, void *address, int count,
			   void (*callback) (void *), void *callback_par)
{
	/* Parameters:
	 *   port: port in destination node
	 *   dest: destination node
	 *   tag: user tag
	 *   req: request returned by the send
	 *   formatp: format of the sent data
	 *   address: address of the sent data
	 *     count: number of data items sent
	 */

	a0tError err;
	a0tDatatype datatype;

	a0Debug(("_a0IFSendCallback: dest=%d, tag=%d\n", dest, tag));

	switch (formatp->type) {

	case A0Map:		/* an MPI datatype map */

		/* Convert format to datatype */
		if (err = _a0GetMapDatatype(formatp, &datatype))
			return err;

		return _a0ISendCallback(port, dest, tag, req,
		       datatype, address, count, callback, callback_par);

	case A0Xdr:
		return _a0XdrISendCallback(port, dest, tag, req,
			formatp, address, count, callback, callback_par);

	default:
		return A0ErrInvFormat;
	}
}


/*************************************************************************
		_a0IFSend - Sends a formatted message (nonblocking)
*************************************************************************/

extern a0tError _a0IFSend(
		     a0tPort * port, int dest, int tag, a0tRequest * req,
			   a0tFormat * formatp, void *address, int count)
{
	return _a0IFSendCallback(port, dest, tag, req,
				 formatp, address, count, 0, 0);
}


/*************************************************************************
		_a0FSend - Sends a formatted message (blocking)
*************************************************************************/

extern a0tError _a0FSend(
		     a0tPort * port, int dest, int tag, a0tRequest * req,
			   a0tFormat * formatp, void *address, int count)
{
	a0tError err;
	a0tRequest req_aux;

	if (!req)
		req = &req_aux;

	if (err = _a0IFSendCallback(port, dest, tag, req,
				    formatp, address, count, 0, 0))
		return err;
	return _a0WaitRequest(req);
}


/*************************************************************************
_a0IFReceiveCallback - Receives formatted message (nonblocking) w/callback
*************************************************************************/

extern a0tError _a0IFReceiveCallback(
		      a0tPort * port, int src, int tag, a0tRequest * req,
			   a0tFormat * formatp, void *address, int count,
			   void (*callback) (void *), void *callback_par)
{
	/* Parameters:
	 *   port: port in destination node
	 *   src: source node
	 *   tag: user tag
	 *   req: request returned by the send
	 *   formatp: format of the received data
	 *   address: address of the received data
	 *     count: number of data items received
	 */

	a0tError err;
	a0tDatatype datatype;

	a0Debug(("_a0IFReceiveCallback: src=%d, tag=%d\n", src, tag));

	switch (formatp->type) {

	case A0Map:		/* an MPI datatype map */

		/* Convert format to datatype */
		if (err = _a0GetMapDatatype(formatp, &datatype))
			return err;

		return _a0IReceiveCallback(port, src, tag, req,
		       datatype, address, count, callback, callback_par);

	case A0Xdr:
		return _a0XdrIReceiveCallback(port, src, tag, req,
			formatp, address, count, callback, callback_par);

	default:
		return A0ErrInvFormat;
	}
}


/*************************************************************************
	_a0IFReceive - Receives a formatted message (non-blocking)
*************************************************************************/

extern a0tError _a0IFReceive(
		      a0tPort * port, int src, int tag, a0tRequest * req,
			   a0tFormat * formatp, void *address, int count)
{
	return _a0IFReceiveCallback(port, src, tag, req,
				    formatp, address, count, 0, 0);
}


/*************************************************************************
		_a0FReceive - Receives a formatted message (blocking)
*************************************************************************/

extern a0tError _a0FReceive(
		      a0tPort * port, int src, int tag, a0tRequest * req,
			   a0tFormat * formatp, void *address, int count)
{
	a0tError err;
	a0tRequest req_aux;

	if (!req)
		req = &req_aux;

	if (err = _a0IFReceiveCallback(port, src, tag, req,
				       formatp, address, count, 0, 0))
		return err;
	return _a0WaitRequest(req);
}


/*************************************************************************
		_a0FPack - packs formatted into a buffer
*************************************************************************/

a0tError _a0FPack(a0tBuffer * buffer,
		  a0tFormat * formatp, void *address, int count)
{
	a0tError ret;

	/*
	 * _a0XdrISend creates a buffer of type A0XdrBufferType and calls
	 * the user defined packing function. This function packs all
	 * parts of an xdr data using _a0FPack function recursively. So,
	 * if buffer type here is A0XdrBufferType, this pack is not
	 * effectively done. Only the size of the packed data is
	 * considered, to calculate the buffer size to allocate. Later on,
	 * when the user function returns, _a0XdrISend can allocate a
	 * buffer, recall the user defined function to pack it and 
	 * finally send it.
	 */

	/* Similar behavior happens with pipes. See _a0IFPut. */

	a0Debug(("_a0FPack: fmt=%x add=%x cnt=%x\n", formatp, address, count));

	if (buffer->bufftype == A0XdrBufferType)
		return _a0AddXdrSize(buffer, formatp, address, count);


	switch (formatp->type) {

	case A0Map:
		return _a0MapPack(buffer, formatp, address, count);

	case A0Xdr:
		return _a0XdrPack(buffer, formatp, address, count);

	default:
		return A0ErrInvFormat;
		break;
	}
}


/*************************************************************************
		_a0FUnpack - unpacks formatted from a buffer
*************************************************************************/

a0tError _a0FUnpack(a0tBuffer * buffer,
		    a0tFormat * formatp, void *address, int count)
{

	a0tError ret;

	a0Debug(("_a0FUnpack: fmt=%x add=%x cnt=%x\n", formatp, address, count));

	switch (formatp->type) {

	case A0Map:
		return _a0MapUnpack(buffer, formatp, address, count);

	case A0Xdr:
		return _a0XdrUnpack(buffer, formatp, address, count);

	default:
		return A0ErrInvFormat;

	}
}


/*************************************************************************
  _a0FPackPosition - packs formatted into a buffer at a given position 
*************************************************************************/

a0tError _a0FPackPosition(a0tBuffer * buffer,
			  long *positionp,
			  a0tFormat * format, void *address, int count)
{
	a0tError ret;

	/*
	 * output: value pointed by positionp is set to the position for the
	 *    next pack instruction
	 * side-effect: internal position of the buffer has the same value. 
	 *    This can be used to next do successive pack.
	 */

	/* Change the position of the buffer, before doing a pack
	   without position. 
	 */
	ret = _a0SetBufferPosition(buffer, *positionp);
	if (ret != A0ErrOk)
		return ret;

	ret = _a0FPack(buffer, format, address, count);
	if (ret != A0ErrOk)
		return ret;

	_a0GetBufferPosition(buffer, positionp);
	return A0ErrOk;
}


/*************************************************************************
  _a0FUnpackPosition - unpacks formatted into a buffer at a given position 
*************************************************************************/

a0tError _a0FUnpackPosition(a0tBuffer * buffer,
			    long *positionp,
			    a0tFormat * format, void *address, int count)
{
	a0tError ret;

	ret = _a0SetBufferPosition(buffer, *positionp);
	if (ret != A0ErrOk)
		return ret;

	ret = _a0FUnpack(buffer, format, address, count);
	if (ret != A0ErrOk)
		return ret;

	_a0GetBufferPosition(buffer, positionp);
	return A0ErrOk;
}
